@@ -27,13 +27,30 @@ import ch.kuramo.javie.api.IAnimatableValue;
 import ch.kuramo.javie.api.IAudioBuffer;
 import ch.kuramo.javie.api.IObjectArray;
 
+/**
+ * オーディオエフェクト用のエフェクトコンテキストです。
+ */
 public interface IAudioEffectContext extends IEffectContext {
 
+	/**
+	 * 引数 avalue の現在時刻からの値を、エフェクトコンテキストのサンプルフレーム数と同じ数だけ取得します。
+	 * 
+	 * @param avalue {@link ch.kuramo.javie.api.IAnimatableDouble} などのアニメーション可能プロパティ
+	 * @return 引数 avalue の現在時刻からの値の配列をラップした <code>IObjectArray</code> オブジェクト
+	 */
 	<V> IObjectArray<V> values(IAnimatableValue<V> avalue);
 
+	/**
+	 * @deprecated このメソッドは、将来、変更または廃止される可能性があります。
+	 */
+	@Deprecated
 	<V> IObjectArray<V> values(IAnimatableValue<V> avalue, int animationRate);
 
-
+	/**
+	 * ひとつ前のオーディオエフェクトまで処理を実行します。
+	 * 
+	 * @return ひとつ前のオーディオエフェクトまで処理を実行した結果のオーディオデータ
+	 */
 	IAudioBuffer doPreviousEffect();
 
 }
