@@ -23,13 +23,34 @@
 
 package ch.kuramo.javie.api.services;
 
+import ch.kuramo.javie.api.ICamera;
 import ch.kuramo.javie.api.IVideoBuffer;
 import ch.kuramo.javie.api.VideoBounds;
 
+/**
+ * ビデオエフェクト用のエフェクトコンテキストです。
+ */
 public interface IVideoEffectContext extends IEffectContext {
 
+	/**
+	 * ひとつ前のビデオエフェクトまで処理を実行した場合に得られるビデオデータの位置とサイズを返します。
+	 * 
+	 * @return ひとつ前のビデオエフェクトまで処理を実行した場合に得られるビデオデータの位置とサイズ
+	 */
 	VideoBounds getPreviousBounds();
 
+	/**
+	 * ひとつ前のビデオエフェクトまで処理を実行します。
+	 * 
+	 * @return ひとつ前のビデオエフェクトまで処理を実行した結果のビデオデータ
+	 */
 	IVideoBuffer doPreviousEffect();
 
+	/**
+	 * コンポジションカメラ (現在時刻における、コンポジションのアクティブなカメラ) を取得します。
+	 * 
+	 * @return コンポジションカメラを表す <code>ICamera</code> オブジェクト
+	 */
+	ICamera getCamera();
+
 }
