@@ -23,6 +23,8 @@
 
 package ch.kuramo.javie.app;
 
+import java.util.Collections;
+import java.util.Comparator;
 import java.util.List;
 import java.util.Map;
 
@@ -292,7 +294,7 @@ public class ApplicationActionBarAdvisor extends ActionBarAdvisor {
 			for (IConfigurationElement cfgElem : ext.getConfigurationElements()) {
 				if ("effect-category".equals(cfgElem.getName())) {
 					String category = cfgElem.getAttribute("category");
-					String name = cfgElem.getAttribute("name");
+					String name = cfgElem.getAttribute("name").replaceAll("&", "&&");
 
 					MenuManager subMenu = new MenuManager(name);
 					subMenus.put(category, subMenu);
@@ -301,8 +303,14 @@ public class ApplicationActionBarAdvisor extends ActionBarAdvisor {
 			}
 		}
 
+		Collections.sort(effectCategorySubMenus, new Comparator<MenuManager>() {
+			public int compare(MenuManager o1, MenuManager o2) {
+				return o1.getMenuText().compareTo(o2.getMenuText());
+			}
+		});
+
 		for (EffectDescriptor ed : effectRegistry.getEffectDescriptors()) {
-			RetargetAction action = new RetargetAction("EFFECT." + ed.getType(), ed.getLabel());
+			RetargetAction action = new RetargetAction("EFFECT." + ed.getType(), ed.getLabel().replaceAll("&", "&&"));
 			//register(action);
 			window.getPartService().addPartListener(action);
 
@@ -313,6 +321,12 @@ public class ApplicationActionBarAdvisor extends ActionBarAdvisor {
 				defaultCategoryEffectActions.add(action);
 			}
 		}
+
+		Collections.sort(defaultCategoryEffectActions, new Comparator<Action>() {
+			public int compare(Action o1, Action o2) {
+				return o1.getText().compareTo(o2.getText());
+			}
+		});
 	}
 
 	protected void fillMenuBar(IMenuManager menuBar) {
