@@ -39,14 +39,13 @@ import org.slf4j.LoggerFactory;
 
 import ch.kuramo.javie.api.AudioMode;
 import ch.kuramo.javie.api.IArray;
+import ch.kuramo.javie.api.IAudioBuffer;
 import ch.kuramo.javie.api.Resolution;
 import ch.kuramo.javie.api.Time;
 import ch.kuramo.javie.api.services.IArrayPools;
 import ch.kuramo.javie.app.InjectorHolder;
-import ch.kuramo.javie.core.AudioBuffer;
 import ch.kuramo.javie.core.Composition;
 import ch.kuramo.javie.core.CompositionItem;
-import ch.kuramo.javie.core.FrameDuration;
 import ch.kuramo.javie.core.MediaInput;
 import ch.kuramo.javie.core.MediaItem;
 import ch.kuramo.javie.core.services.RenderContext;
@@ -155,7 +154,7 @@ public class AudioThread extends Thread {
 					continue;
 				}
 
-				AudioBuffer ab = null;
+				IAudioBuffer ab = null;
 
 				if (!_muted) {
 					_context.reset();
@@ -180,8 +179,6 @@ public class AudioThread extends Thread {
 							PlayerLock.readLock().unlock();
 						}
 					} else {
-						_context.setVideoFrameDuration(FrameDuration.FPS_29_97);
-
 						try {
 							ab = input.getAudioChunk(time);
 						} catch (Exception e) {
@@ -198,7 +195,7 @@ public class AudioThread extends Thread {
 				}
 
 				// 現在のところ、要求したフレーム数分だけ必ず返す事になっているので、
-				// frameCountの値の分だけ時間を進めればよいが、一応 AudioBuffer のフレーム数を見て時間を進める。
+				// frameCountの値の分だけ時間を進めればよいが、一応 IAudioBuffer のフレーム数を見て時間を進める。
 				nextTime = time.add(Time.fromFrameNumber(
 						(ab != null) ? ab.getFrameCount() : frameCount, audioMode.sampleDuration));
 
@@ -259,7 +256,7 @@ public class AudioThread extends Thread {
 		return !_finished;
 	}
 
-	private void write(AudioBuffer ab) {
+	private void write(IAudioBuffer ab) {
 		AudioMode audioMode = ab.getAudioMode();
 		int dataLen = ab.getDataLength();
 		int lineDataLen = _line.getFormat().getFrameSize() * ab.getFrameCount();
