@@ -257,7 +257,7 @@ public class MediaPlayer extends Composite implements PlayerThreadListener, Play
 		_duration = input.getDuration();
 
 		if (_duration != null) {
-			if (input.isVideoAvailable()) {
+			if (input.isVideoAvailable() && input.getVideoFrameDuration().timeValue > 0) {
 				_frameDuration = input.getVideoFrameDuration();
 			} else {
 				// TODO これだとデュレーションが12時間25分以上になると問題が発生するはず。
@@ -479,9 +479,11 @@ public class MediaPlayer extends Composite implements PlayerThreadListener, Play
 
 	public void handleProjectEvent(ProjectEvent event) {
 		switch (event.type) {
+			case ITEM_UPDATE:
 			case COMPOSITION_SETTINGS_CHANGE:
 			case COMPOSITION_PROPERTY_CHANGE:
 			case LAYER_PROPERTY_CHANGE:
+			case LAYER_ITEM_CHANGE:
 			case LAYERS_ADD:
 			case LAYERS_REMOVE:
 			case LAYERS_REORDER:
