@@ -61,6 +61,7 @@ import org.slf4j.LoggerFactory;
 import ch.kuramo.javie.api.AudioMode;
 import ch.kuramo.javie.api.Color;
 import ch.kuramo.javie.api.ColorMode;
+import ch.kuramo.javie.api.IVideoBuffer;
 import ch.kuramo.javie.api.Resolution;
 import ch.kuramo.javie.api.Size2i;
 import ch.kuramo.javie.api.Time;
@@ -76,7 +77,6 @@ import ch.kuramo.javie.core.MediaInputPlaceholder;
 import ch.kuramo.javie.core.MediaItem;
 import ch.kuramo.javie.core.SolidColorItem;
 import ch.kuramo.javie.core.TimeCode;
-import ch.kuramo.javie.core.VideoBuffer;
 import ch.kuramo.javie.core.misc.AtiIntelLock;
 import ch.kuramo.javie.core.services.GLGlobal;
 import ch.kuramo.javie.core.services.RenderContext;
@@ -111,7 +111,7 @@ public class VideoCanvas extends Thread implements GLEventListener {
 
 	private final AtomicReference<Object[]> _videoBufferAndTimeRef = new AtomicReference<Object[]>();
 
-	private final Queue<VideoBuffer> _oldVideoBuffers = new ConcurrentLinkedQueue<VideoBuffer>();
+	private final Queue<IVideoBuffer> _oldVideoBuffers = new ConcurrentLinkedQueue<IVideoBuffer>();
 
 	private volatile Time _frameDuration;
 
@@ -157,7 +157,7 @@ public class VideoCanvas extends Thread implements GLEventListener {
 	@Inject
 	private GLGlobal _glGlobal;
 
-	private final boolean _intel;
+//	private final boolean _intel;
 
 	private final AtiIntelLock _atiIntelLock;
 
@@ -194,7 +194,7 @@ public class VideoCanvas extends Thread implements GLEventListener {
 
 		setZoom(1.0);
 
-		_intel = _glGlobal.isIntel();
+//		_intel = _glGlobal.isIntel();
 		_atiIntelLock = AtiIntelLock.get(_glGlobal);
 	}
 
@@ -442,14 +442,14 @@ public class VideoCanvas extends Thread implements GLEventListener {
 				return;
 			}
 
-			VideoBuffer vb = (VideoBuffer) vbAndTime[0];
+			IVideoBuffer vb = (IVideoBuffer) vbAndTime[0];
 
 			int[] vp = _viewport;
 			gl.glViewport(vp[0], vp[1], vp[2], vp[3]);
 
-			Size2i imageSize = vb.getImageSize();
-			int w = imageSize.width;
-			int h = imageSize.height;
+			VideoBounds bounds = vb.getBounds();
+			int w = bounds.width;
+			int h = bounds.height;
 
 			gl.glMatrixMode(GL2.GL_PROJECTION);
 			gl.glLoadIdentity();
@@ -484,7 +484,7 @@ public class VideoCanvas extends Thread implements GLEventListener {
 			_atiIntelLock.unlock();
 
 			if (vbAndTime != null && !_videoBufferAndTimeRef.compareAndSet(null, vbAndTime)) {
-				_oldVideoBuffers.add((VideoBuffer) vbAndTime[0]);
+				_oldVideoBuffers.add((IVideoBuffer) vbAndTime[0]);
 			}
 
 			_displaying = false;
@@ -510,7 +510,7 @@ public class VideoCanvas extends Thread implements GLEventListener {
 
 		} else if (_mediaItem instanceof SolidColorItem) {
 			Color color = ((SolidColorItem) _mediaItem).getColor();
-			info.add(String.format("Solid Color: %d x %d, #%02x%02x%02x", bounds.width, bounds.height,
+			info.add(String.format("Solid Color: %d x %d, R:%d G:%d B:%d", bounds.width, bounds.height,
 					(int)(color.r*255), (int)(color.g*255), (int)(color.b*255)));
 
 		} else if (_mediaItem.getMediaInput() instanceof MediaInputPlaceholder) {
@@ -519,7 +519,7 @@ public class VideoCanvas extends Thread implements GLEventListener {
 		} else if (_mediaItem instanceof ImageSequenceItem) {
 			info.add(String.format("Image Sequence: %d x %d", bounds.width, bounds.height));
 
-		} else if (_mediaInput.getVideoFrameDuration() != null) {
+		} else if (frameDuration != null) {
 			info.add(String.format("Video: %d x %d", bounds.width, bounds.height));
 
 		} else {
@@ -527,10 +527,14 @@ public class VideoCanvas extends Thread implements GLEventListener {
 		}
 
 		if (frameDuration != null) {
-			Time duration = _mediaInput.getDuration();
-			info.add(String.format("Frame: %d / %d", time.toFrameNumber(frameDuration), duration.toFrameNumber(frameDuration)));
-			info.add(String.format("Time: %s / %s", TimeCode.toTimeCode(time, frameDuration), TimeCode.toTimeCode(duration, frameDuration)));
-			info.add(String.format("FPS: %.2f / %.2f", 1/_frameDurationAvg.getSampledAverage(), 1/frameDuration.toSecond()));
+			if (frameDuration.timeValue > 0) {
+				Time duration = _mediaInput.getDuration();
+				info.add(String.format("Frame: %d / %d", time.toFrameNumber(frameDuration), duration.toFrameNumber(frameDuration)));
+				info.add(String.format("Time: %s / %s", TimeCode.toTimeCode(time, frameDuration), TimeCode.toTimeCode(duration, frameDuration)));
+				info.add(String.format("FPS: %.2f / %.2f", 1/_frameDurationAvg.getSampledAverage(), 1/frameDuration.toSecond()));
+			} else {
+				info.add(String.format("FPS: %.2f", 1/_frameDurationAvg.getSampledAverage()));
+			}
 		}
 
 		int infoWidth = 0;
@@ -586,11 +590,8 @@ public class VideoCanvas extends Thread implements GLEventListener {
 				_context.reset();
 				_context.setVideoResolution(resolution);
 				_context.setColorMode(ColorMode.RGBA8);
-				_context.setVideoFrameDuration(TIME0);
-				_context.setAudioMode(audioMode);
-				_context.setAudioAnimationRate(audioMode.sampleRate/100);
 
-				VideoBuffer vb = null;
+				IVideoBuffer vb = null;
 				try {
 					_atiIntelLock.lock();
 					try {
@@ -642,7 +643,8 @@ public class VideoCanvas extends Thread implements GLEventListener {
 
 				Time frameDuration = _frameDuration;
 				if (frameDuration == null) {
-					frameDuration = _mediaInput.getVideoFrameDuration();
+					Time fd = _mediaInput.getVideoFrameDuration();
+					frameDuration = (fd.timeValue > 0) ? fd : FrameDuration.FPS_59_94;
 				}
 
 				boolean seek = (timeAndBase[1] != prevBase);
@@ -697,7 +699,7 @@ public class VideoCanvas extends Thread implements GLEventListener {
 				_context.setAudioMode(audioMode);
 				_context.setAudioAnimationRate(audioMode.sampleRate/100);
 
-				VideoBuffer vb = null;
+				IVideoBuffer vb = null;
 
 				if (_mediaItem instanceof CompositionItem) {
 					PlayerLock.readLock().lock();
@@ -756,7 +758,7 @@ public class VideoCanvas extends Thread implements GLEventListener {
 						Object[] old = _videoBufferAndTimeRef.getAndSet(new Object[] { vb, frameTime });
 						_canvas.repaint();
 						if (old != null) {
-							((VideoBuffer) old[0]).dispose();
+							((IVideoBuffer) old[0]).dispose();
 						}
 
 						fireThreadRender(frameTime);
@@ -791,7 +793,7 @@ public class VideoCanvas extends Thread implements GLEventListener {
 
 			Object[] vbAndTime = _videoBufferAndTimeRef.getAndSet(null);
 			if (vbAndTime != null) {
-				((VideoBuffer) vbAndTime[0]).dispose();
+				((IVideoBuffer) vbAndTime[0]).dispose();
 			}
 			disposeOldVideoBuffers();
 
@@ -836,11 +838,11 @@ public class VideoCanvas extends Thread implements GLEventListener {
 		cleanup();
 	}
 
-	private void glFinish(GL2 gl, VideoBuffer vb) {
-		if (!_intel) {
-			gl.glFinish();
-			return;
-		}
+	private void glFinish(GL2 gl, IVideoBuffer vb) {
+//		if (!_intel) {
+//			gl.glFinish();
+//			return;
+//		}
 
 		// Intel GMAの場合、カラーバッファに何もアタッチされていないとglFinishに失敗する。
 		// (GMA X4500で確認) 
@@ -854,7 +856,7 @@ public class VideoCanvas extends Thread implements GLEventListener {
 				GL2.GL_COLOR_ATTACHMENT0, GL2.GL_TEXTURE_RECTANGLE, 0, 0);
 	}
 
-	private void setTexParametersForDisplay(VideoBuffer vb, Resolution resolution, GL2 gl) {
+	private void setTexParametersForDisplay(IVideoBuffer vb, Resolution resolution, GL2 gl) {
 		gl.glBindTexture(GL2.GL_TEXTURE_RECTANGLE, vb.getTexture());
 
 		// フル解像度のときは、拡大したときにテクスチャの補間が効かないようにする。
@@ -958,8 +960,8 @@ public class VideoCanvas extends Thread implements GLEventListener {
 	}
 
 	private void disposeOldVideoBuffers() {
-		for (Iterator<VideoBuffer> it = _oldVideoBuffers.iterator(); it.hasNext(); ) {
-			VideoBuffer old = it.next();
+		for (Iterator<IVideoBuffer> it = _oldVideoBuffers.iterator(); it.hasNext(); ) {
+			IVideoBuffer old = it.next();
 			old.dispose();
 			it.remove();
 		}
