@@ -168,6 +168,10 @@ public class NewLayerCompositionOperation extends ProjectOperation {
 		}
 	}
 
+	public String getLayerCompositionId() {
+		return _compId;
+	}
+
 	public String getCompositionItemId() {
 		return _itemId;
 	}
