@@ -35,6 +35,7 @@ import ch.kuramo.javie.core.Composition;
 import ch.kuramo.javie.core.Effect;
 import ch.kuramo.javie.core.EffectableLayer;
 import ch.kuramo.javie.core.Item;
+import ch.kuramo.javie.core.ItemLayer;
 import ch.kuramo.javie.core.Layer;
 import ch.kuramo.javie.core.LayerComposition;
 
@@ -51,6 +52,7 @@ public class ProjectEvent extends EventObject {
 		FILE_CHANGE,
 
 		ITEM_NAME_CHANGE,
+		ITEM_UPDATE,
 		ITEMS_ADD,
 		ITEMS_REMOVE,
 		ITEMS_REPARENT,
@@ -59,6 +61,7 @@ public class ProjectEvent extends EventObject {
 		COMPOSITION_PROPERTY_CHANGE,
 
 		LAYER_PROPERTY_CHANGE,
+		LAYER_ITEM_CHANGE,
 		LAYERS_ADD,
 		LAYERS_REMOVE,
 		LAYERS_REORDER,
@@ -138,6 +141,11 @@ public class ProjectEvent extends EventObject {
 		return new ProjectEvent(pm, Type.ITEM_NAME_CHANGE, item, null, null, -1, null);
 	}
 
+	public static ProjectEvent createItemUpdateEvent(ProjectManager pm, Item item) {
+		pm.checkItem(item);
+		return new ProjectEvent(pm, Type.ITEM_UPDATE, item, null, null, -1, null);
+	}
+
 	private static <T> Set<T> toUnmodifiableSet(Collection<? extends T> c) {
 		if (c == null) {
 			return Collections.<T>emptySet();
@@ -205,6 +213,10 @@ public class ProjectEvent extends EventObject {
 		return new ProjectEvent(pm, Type.LAYER_PROPERTY_CHANGE, null, pm.checkLayer(layer), layer, -1, property);
 	}
 
+	public static ProjectEvent createLayerItemChangeEvent(ProjectManager pm, ItemLayer<?> layer) {
+		return new ProjectEvent(pm, Type.LAYER_ITEM_CHANGE, null, pm.checkLayer(layer), layer, -1, null);
+	}
+
 	private static void checkLayers(ProjectManager pm, LayerComposition comp, Collection<? extends Layer> layers) {
 		for (Layer l : layers) {
 			if (pm.checkLayer(l) != comp) {
@@ -278,10 +290,13 @@ public class ProjectEvent extends EventObject {
 	}
 
 	public static Item getItem(ProjectEvent event) {
-		if (event.type != Type.ITEM_NAME_CHANGE) {
-			throw new IllegalArgumentException("event type is not ITEM_NAME_CHANGE: type=" + event.type);
+		switch (event.type) {
+			case ITEM_NAME_CHANGE:
+			case ITEM_UPDATE:
+				return (Item) event.data;
+			default:
+				throw new IllegalArgumentException("event type is neither ITEM_NAME_CHANGE nor ITEM_UPDATE: type=" + event.type);
 		}
-		return (Item) event.data;
 	}
 
 	private static Set<Item> getItems(ProjectEvent event, int index) {
