@@ -63,6 +63,7 @@ import ch.kuramo.javie.core.Composition;
 import ch.kuramo.javie.core.EffectableLayer;
 import ch.kuramo.javie.core.Folder;
 import ch.kuramo.javie.core.Item;
+import ch.kuramo.javie.core.ItemLayer;
 import ch.kuramo.javie.core.JavieRuntimeException;
 import ch.kuramo.javie.core.Layer;
 import ch.kuramo.javie.core.LayerComposition;
@@ -319,6 +320,10 @@ public class ProjectManager implements IOperationHistoryListener {
 		fireProjectEvent(ProjectEvent.createItemNameChangeEvent(this, item));
 	}
 
+	void fireItemUpdate(Item item) {
+		fireProjectEvent(ProjectEvent.createItemUpdateEvent(this, item));
+	}
+
 	void fireItemsAdd(Collection<? extends Item> items, Collection<? extends Item> relatedItems) {
 		fireProjectEvent(ProjectEvent.createItemsAddEvent(this, items, relatedItems));
 	}
@@ -343,6 +348,10 @@ public class ProjectManager implements IOperationHistoryListener {
 		fireProjectEvent(ProjectEvent.createLayerPropertyChangeEvent(this, layer, property));
 	}
 
+	void fireLayerItemChange(ItemLayer<?> layer) {
+		fireProjectEvent(ProjectEvent.createLayerItemChangeEvent(this, layer));
+	}
+
 	void fireLayersAdd(LayerComposition comp, Collection<? extends Layer> layers) {
 		fireProjectEvent(ProjectEvent.createLayersAddEvent(this, comp, layers));
 	}
