@@ -28,20 +28,25 @@ import org.eclipse.core.runtime.IAdaptable;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Status;
+import org.mozilla.javascript.Context;
+import org.mozilla.javascript.ContextFactory;
+import org.mozilla.javascript.Scriptable;
+import org.mozilla.javascript.ScriptableObject;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
-import ch.kuramo.javie.api.Size2i;
 import ch.kuramo.javie.api.Vec2d;
 import ch.kuramo.javie.api.Vec3d;
 import ch.kuramo.javie.api.VideoBounds;
 import ch.kuramo.javie.app.InjectorHolder;
 import ch.kuramo.javie.app.PropertyUtil;
 import ch.kuramo.javie.core.AnimatableDouble;
+import ch.kuramo.javie.core.AnimatableLayerReference;
 import ch.kuramo.javie.core.AnimatableString;
 import ch.kuramo.javie.core.AnimatableValue;
 import ch.kuramo.javie.core.AnimatableVec2d;
 import ch.kuramo.javie.core.AnimatableVec3d;
 import ch.kuramo.javie.core.CameraLayer;
-import ch.kuramo.javie.core.Composition;
 import ch.kuramo.javie.core.Effect;
 import ch.kuramo.javie.core.EffectableLayer;
 import ch.kuramo.javie.core.Keyframe;
@@ -64,6 +69,9 @@ import com.google.inject.Inject;
 
 public class ResetAnimatableValuesOperation extends ProjectOperation {
 
+	private static final Logger _logger = LoggerFactory.getLogger(ResetAnimatableValuesOperation.class);
+
+
 	private final String _compId;
 
 	private final Object[][] _properties;
@@ -79,6 +87,9 @@ public class ResetAnimatableValuesOperation extends ProjectOperation {
 	@Inject
 	private FontList _fontList;
 
+	@Inject
+	private ContextFactory _jsContextFactory;
+
 
 	public ResetAnimatableValuesOperation(ProjectManager pm, LayerComposition comp, Object[][] properties) {
 		super(pm, "リセット");
@@ -160,12 +171,7 @@ public class ResetAnimatableValuesOperation extends ProjectOperation {
 				if (effectIndex == -1) {
 					resetLayerProperty(comp, layer, property, avalue);
 				} else {
-					avalue.reset();
-
-					if (layer instanceof MediaLayer && avalue instanceof AnimatableVec2d
-							&& Boolean.parseBoolean(propDesc.getString("initiallyCenter"))) {
-						center((AnimatableVec2d) avalue, (MediaLayer) layer, comp);
-					}
+					resetEffectProperty(comp, layer, propDesc, avalue);
 				}
 				if (pm != null) {
 					data[i] = new Object[] { layer, effectIndex, property, null };
@@ -194,21 +200,6 @@ public class ResetAnimatableValuesOperation extends ProjectOperation {
 		}
 	}
 
-	private void center(AnimatableVec2d avalue, MediaLayer layer, Composition comp) {
-		if (LayerNature.isCTCR(layer)) {
-			Size2i size = comp.getSize();
-			avalue.reset(new Vec2d(size.width/2.0, size.height/2.0));
-		} else {
-			MediaInput input = layer.getMediaInput();
-			if (input != null) {
-				VideoBounds bounds = input.getVideoFrameBounds();
-				if (bounds != null) {
-					avalue.reset(new Vec2d(bounds.x + bounds.width/2.0, bounds.y + bounds.height/2.0));
-				}
-			}
-		}
-	}
-
 	private void resetLayerProperty(
 			LayerComposition comp, Layer layer, String property, AnimatableValue<?> avalue) {
 
@@ -272,6 +263,12 @@ public class ResetAnimatableValuesOperation extends ProjectOperation {
 		avalue.reset();
 	}
 
+	private void resetEffectProperty(
+			LayerComposition comp, Layer layer, PropertyDescriptor propDesc, AnimatableValue<?> avalue) {
+
+		resetEffectProperty(comp, layer, propDesc, avalue, _jsContextFactory);
+	}
+
 	// TODO ThreeDSwitchOperation 等にほぼ同じメソッドがあるので整理する。
 	private <A extends AnimatableValue<V>, V> void restore(String data, A dst) throws ExecutionException {
 		try {
@@ -281,7 +278,7 @@ public class ResetAnimatableValuesOperation extends ProjectOperation {
 			dst.reset(src.getStaticValue());
 			dst.setExpression(src.getExpression());
 
-			for (Keyframe<V> kf : src.getKeyframes()) {
+			for (Keyframe<V> kf : src.getKeyframeMap().values()) {
 				dst.putKeyframe(kf);
 			}
 
@@ -290,4 +287,52 @@ public class ResetAnimatableValuesOperation extends ProjectOperation {
 		}
 	}
 
+	public static void resetEffectProperty(
+			LayerComposition comp, Layer layer,
+			PropertyDescriptor propDesc, AnimatableValue<?> avalue,
+			ContextFactory jsContextFactory) {
+
+		String resetScript = propDesc.getString("reset");
+		if (resetScript == null) {
+			if (avalue instanceof AnimatableLayerReference) {
+				((AnimatableLayerReference) avalue).reset(layer.getId());
+			} else {
+				avalue.reset();
+			}
+			return;
+		}
+
+		VideoBounds videoBounds = null;
+		if (LayerNature.isCTCR(layer)) {
+			videoBounds = new VideoBounds(comp.getSize());
+		} else if (layer instanceof MediaLayer) {
+			MediaInput input = ((MediaLayer) layer).getMediaInput();
+			if (input != null) {
+				videoBounds = input.getVideoFrameBounds();
+			}
+		}
+		if (videoBounds == null) {
+			videoBounds = new VideoBounds(0, 0);
+		}
+
+		Context jsContext = jsContextFactory.enterContext();
+		try {
+			Scriptable globalScope = jsContext.initStandardObjects();
+			ScriptableObject.putProperty(globalScope, "videoBounds", Context.javaToJS(videoBounds, globalScope));
+			Object jsValue = jsContext.evaluateString(globalScope, resetScript, propDesc.getName(), 1, null);
+
+			@SuppressWarnings("unchecked")
+			AnimatableValue<Object> tmp = (AnimatableValue<Object>) avalue;
+			tmp.reset(tmp.jsToJava(jsValue));
+			return;
+
+		} catch (Exception e) {
+			_logger.warn("error resetting AnimatableValue", e);
+		} finally {
+			Context.exit();
+		}
+
+		avalue.reset();
+	}
+
 }
