@@ -118,7 +118,7 @@ public class ShiftKeyframesOperation extends ProjectOperation {
 
 			@SuppressWarnings("unchecked")
 			AnimatableValue<Object> tmp = (AnimatableValue<Object>) newAvalue;
-			tmp.putKeyframe(kf.time.add(_shiftTime), kf.value, kf.interpolation);
+			tmp.putKeyframe(_shiftTime.add(kf.time), kf.value, kf.interpolation);
 		}
 
 		for (int i = 0, n = newAvalues.length; i < n; ++i) {
@@ -167,7 +167,7 @@ public class ShiftKeyframesOperation extends ProjectOperation {
 					restore(_newAvalues[i], avalue);
 				}
 				if (pm != null) {
-					data[i] = new Object[] { layer, effectIndex, property, kf.time.add(_shiftTime) };
+					data[i] = new Object[] { layer, effectIndex, property, _shiftTime.add(kf.time) };
 				}
 			} else {
 				if (_savedAvalues[i] != null) {
@@ -193,7 +193,7 @@ public class ShiftKeyframesOperation extends ProjectOperation {
 			dst.reset(src.getStaticValue());
 			dst.setExpression(src.getExpression());
 
-			for (Keyframe<V> kf : src.getKeyframes()) {
+			for (Keyframe<V> kf : src.getKeyframeMap().values()) {
 				dst.putKeyframe(kf);
 			}
 
