@@ -140,8 +140,8 @@ public class ShiftLayerTimesOperation extends ProjectOperation {
 
 	private <V> void shiftKeyframes(AnimatableValue<V> avalue) {
 		List<Keyframe<V>> list = Util.newList();
-		for (Keyframe<V> kf : avalue.getKeyframes()) {
-			list.add(new Keyframe<V>(kf.time.add(_deltaTime), kf.value, kf.interpolation));
+		for (Keyframe<V> kf : avalue.getKeyframeMap().values()) {
+			list.add(new Keyframe<V>(_deltaTime.add(kf.time), kf.value, kf.interpolation));
 		}
 
 		avalue.clearKeyframes(avalue.getStaticValue());
@@ -159,7 +159,7 @@ public class ShiftLayerTimesOperation extends ProjectOperation {
 			dst.reset(src.getStaticValue());
 			dst.setExpression(src.getExpression());
 
-			for (Keyframe<V> kf : src.getKeyframes()) {
+			for (Keyframe<V> kf : src.getKeyframeMap().values()) {
 				dst.putKeyframe(kf);
 			}
 
@@ -177,9 +177,9 @@ public class ShiftLayerTimesOperation extends ProjectOperation {
 
 		for (Entry e : _entries) {
 			Layer layer = comp.getLayer(e.layerId);
-			layer.setStartTime(e.oldTimes[0].add(_deltaTime));
-			layer.setInPoint(e.oldTimes[1].add(_deltaTime));
-			layer.setOutPoint(e.oldTimes[2].add(_deltaTime));
+			layer.setStartTime(_deltaTime.add(e.oldTimes[0]));
+			layer.setInPoint(_deltaTime.add(e.oldTimes[1]));
+			layer.setOutPoint(_deltaTime.add(e.oldTimes[2]));
 
 			Iterator<String> it = e.oldAvalues.iterator();
 
@@ -282,7 +282,7 @@ public class ShiftLayerTimesOperation extends ProjectOperation {
 			}
 		}
 
-		_deltaTime = _deltaTime.add(newOp._deltaTime);
+		_deltaTime = newOp._deltaTime;
 
 		try {
 			executeOrRedo(monitor, info, project, pm);
