@@ -167,7 +167,7 @@ public class ThreeDSwitchOperation extends ProjectOperation {
 	private void setZ(AnimatableVec3d avalue, double z) {
 		List<Keyframe<Vec3d>> newKeyframes = Util.newList();
 
-		for (Keyframe<Vec3d> kf : avalue.getKeyframes()) {
+		for (Keyframe<Vec3d> kf : avalue.getKeyframeMap().values()) {
 			Vec3d v = kf.value;
 			newKeyframes.add(new Keyframe<Vec3d>(kf.time, new Vec3d(v.x, v.y, z), kf.interpolation));
 		}
@@ -222,7 +222,7 @@ public class ThreeDSwitchOperation extends ProjectOperation {
 			dst.reset(src.getStaticValue());
 			dst.setExpression(src.getExpression());
 
-			for (Keyframe<V> kf : src.getKeyframes()) {
+			for (Keyframe<V> kf : src.getKeyframeMap().values()) {
 				dst.putKeyframe(kf);
 			}
 
