@@ -27,6 +27,7 @@ import java.lang.reflect.Method;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.Comparator;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
@@ -132,9 +133,11 @@ import ch.kuramo.javie.app.actions.AVIOutputAction;
 import ch.kuramo.javie.app.actions.AddEffectAction;
 import ch.kuramo.javie.app.actions.AddExpressionsAction;
 import ch.kuramo.javie.app.actions.KeyframeInterpolationAction;
+import ch.kuramo.javie.app.actions.PrecomposeAction;
 import ch.kuramo.javie.app.actions.QTMovieOutputAction;
 import ch.kuramo.javie.app.actions.RemoveKeyframesAction;
 import ch.kuramo.javie.app.actions.RenameLayerOrEffectAction;
+import ch.kuramo.javie.app.actions.ReplaceLayerItemAction;
 import ch.kuramo.javie.app.actions.ResetAnimatableValuesAction;
 import ch.kuramo.javie.app.actions.ReverseLayerTimeAction;
 import ch.kuramo.javie.app.actions.SequenceOutputAction;
@@ -371,24 +374,30 @@ public class LayerCompositionView extends ViewPart
 	}
 
 	public void handleProjectEvent(ProjectEvent event) {
-		ProjectManager pm = event.getProjectManager();
-		if (pm.getWorkbenchWindow() != getSite().getWorkbenchWindow()) {
+		if (event.getProjectManager() != _projectManager) {
 			return;
 		}
 
 		switch (event.type) {
 			case LAYER_PROPERTY_CHANGE:
+			case LAYER_ITEM_CHANGE:
 			case LAYERS_REMOVE:
 			case EFFECT_PROPERTY_CHANGE:
 			case EFFECTS_REMOVE:
-				_timelineManager.clearKeyframeSelection();
-				_timelineManager.redraw();
+				if (event.composition == _composition) {
+					_timelineManager.clearKeyframeSelection();
+					_timelineManager.redraw();
+				}
 				break;
 		}
 
 		switch (event.type) {
 			case ITEM_NAME_CHANGE:
-				handleItemNameChange(event);
+				handleItemUpdate(event, true);
+				break;
+
+			case ITEM_UPDATE:
+				handleItemUpdate(event, false);
 				break;
 
 			case COMPOSITION_SETTINGS_CHANGE:
@@ -407,6 +416,10 @@ public class LayerCompositionView extends ViewPart
 				handleLayerExpressionChange(event);
 				break;
 
+			case LAYER_ITEM_CHANGE:
+				handleLayerItemChange(event);
+				break;
+
 			case LAYERS_ADD:
 			case LAYERS_REMOVE:
 			case LAYERS_REORDER:
@@ -789,18 +802,27 @@ public class LayerCompositionView extends ViewPart
 
 
 		final Rectangle timeCodeArea = new Rectangle(0, 0, 0, 0);
+		final Time[] newTime = new Time[1];
 
 		_timeCodeComposite.addPaintListener(new PaintListener() {
 			private final int[] underlineDash = new int[] { 2, 2 };
 
 			public void paintControl(PaintEvent e) {
-				if (_time == null || _composition == null) {
+				if (_composition == null) {
 					return;
 				}
 
 				GC gc = e.gc;
 
-				String timeCode = TimeCode.toTimeCode(_time, _composition.getFrameDuration());
+				Time time;
+				if (newTime[0] != null) {
+					gc.setForeground(e.display.getSystemColor(SWT.COLOR_DARK_RED));
+					time = newTime[0];
+				} else {
+					time = _time;
+				}
+
+				String timeCode = TimeCode.toTimeCode(time, _composition.getFrameDuration());
 				Point extent = gc.textExtent(timeCode, SWT.DRAW_TRANSPARENT);
 				Point size = _timeCodeComposite.getSize();
 				int x = size.x - extent.x + (COCOA ? -3 : WIN32 ? -1 : 0);
@@ -889,9 +911,10 @@ public class LayerCompositionView extends ViewPart
 										} else if (!time.before(_composition.getDuration())) {
 											time = _composition.getDuration().subtract(frameDuration);
 										}
-										time = Time.fromFrameNumber(time.toFrameNumber(frameDuration), frameDuration);
+										newTime[0] = Time.fromFrameNumber(time.toFrameNumber(frameDuration), frameDuration);
 
-										update(time);
+										update(newTime[0]);
+										_timeCodeComposite.redraw();
 									}
 
 									break;
@@ -906,14 +929,19 @@ public class LayerCompositionView extends ViewPart
 								display.removeFilter(SWT.MouseUp, this);
 								display.removeFilter(SWT.Deactivate, this);
 								_timelineManager.setWrapMode(true);
+								newTime[0] = null;
 
-								if (e.type == SWT.MouseUp && !_timeCodeComposite.isDisposed() && !dragDetected) {
-									editorCanceled[0] = false;
-									editor.setText(TimeCode.toTimeCode(_time, _composition.getFrameDuration()));
-									editor.selectAll();
-									editor.setVisible(true);
-									if (!editor.setFocus()) {
-										editor.setVisible(false);
+								if (!_timeCodeComposite.isDisposed()) {
+									if (e.type == SWT.MouseUp && !dragDetected) {
+										editorCanceled[0] = false;
+										editor.setText(TimeCode.toTimeCode(_time, _composition.getFrameDuration()));
+										editor.selectAll();
+										editor.setVisible(true);
+										if (!editor.setFocus()) {
+											editor.setVisible(false);
+										}
+									} else {
+										_timeCodeComposite.redraw();
 									}
 								}
 								break;
@@ -1057,6 +1085,8 @@ public class LayerCompositionView extends ViewPart
 		menuMgr.add(deleteAction);
 		menuMgr.add(duplicateAction);
 		menuMgr.add(_splitLayersAction);
+		menuMgr.add(new ReplaceLayerItemAction(this, _treeViewer));
+		menuMgr.add(new PrecomposeAction(this, _treeViewer));
 		menuMgr.add(new ResetAnimatableValuesAction(window, _treeViewer));
 		menuMgr.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
 
@@ -1087,29 +1117,52 @@ public class LayerCompositionView extends ViewPart
 			return;
 		}
 
-		Map<String, MenuManager> subMenus = Util.newMap();
+		Map<String, MenuManager> subMenuMap = Util.newMap();
+		List<MenuManager> subMenuList = Util.newList();
 
 		for (IExtension ext : point.getExtensions()) {
 			for (IConfigurationElement cfgElem : ext.getConfigurationElements()) {
 				if ("effect-category".equals(cfgElem.getName())) {
 					String category = cfgElem.getAttribute("category");
-					String name = cfgElem.getAttribute("name");
+					String name = cfgElem.getAttribute("name").replaceAll("&", "&&");
 
 					MenuManager subMenu = new MenuManager(name);
-					subMenus.put(category, subMenu);
-					effectMenu.add(subMenu);
+					subMenuMap.put(category, subMenu);
+					subMenuList.add(subMenu);
 				}
 			}
 		}
 
+		Collections.sort(subMenuList, new Comparator<MenuManager>() {
+			public int compare(MenuManager o1, MenuManager o2) {
+				return o1.getMenuText().compareTo(o2.getMenuText());
+			}
+		});
+
+		for (MenuManager subMenu : subMenuList) {
+			effectMenu.add(subMenu);
+		}
+
+		List<Action> actions = Util.newList();
+
 		for (AddEffectAction action : effectActions) {
-			MenuManager subMenu = subMenus.get(action.getEffectDescriptor().getCategory());
+			MenuManager subMenu = subMenuMap.get(action.getEffectDescriptor().getCategory());
 			if (subMenu != null) {
 				subMenu.add(action);
 			} else {
-				effectMenu.add(action);
+				actions.add(action);
 			}
 		}
+
+		Collections.sort(actions, new Comparator<Action>() {
+			public int compare(Action o1, Action o2) {
+				return o1.getText().compareTo(o2.getText());
+			}
+		});
+
+		for (Action action : actions) {
+			effectMenu.add(action);
+		}
 	}
 
 	private void setupDragAndDrop() {
@@ -2247,6 +2300,24 @@ public class LayerCompositionView extends ViewPart
 		collectAnimatableValues();
 	}
 
+	private void handleLayerItemChange(ProjectEvent event) {
+		if (event.composition != _composition) {
+			return;
+		}
+
+		_treeViewer.refresh();
+
+		for (TreeItem treeItem : _treeViewer.getTree().getItems()) {
+			LayerElement layerElement = (LayerElement) treeItem.getData();
+			if (layerElement.layer == event.layer) {
+				selectAndReveal(treeItem);
+				break;
+			}
+		}
+
+		collectAnimatableValues();
+	}
+
 	private void handleLayersAddRemoveReorder(ProjectEvent event) {
 		if (event.composition != _composition) {
 			return;
@@ -2282,11 +2353,7 @@ public class LayerCompositionView extends ViewPart
 		collectAnimatableValues();
 	}
 
-	private void handleItemNameChange(ProjectEvent event) {
-		if (event.getProjectManager() != _projectManager) {
-			return;
-		}
-
+	private void handleItemUpdate(ProjectEvent event, boolean checkParent) {
 		Item item = ProjectEvent.getItem(event);
 		if (item == _compItem) {
 			setPartName(_compItem.getName());
@@ -2298,17 +2365,19 @@ public class LayerCompositionView extends ViewPart
 		for (TreeItem treeItem : _treeViewer.getTree().getItems()) {
 			LayerElement element = (LayerElement) treeItem.getData();
 
-			if (element.layer instanceof ItemLayer) {
-				ItemLayer itemLayer = (ItemLayer) element.layer;
+			if (element.layer instanceof ItemLayer<?>) {
+				ItemLayer<?> itemLayer = (ItemLayer<?>) element.layer;
 				if (itemLayer.getItem() == item) {
 					elements.add(element);
 				}
 			}
 
-			Layer parentLayer = _composition.getParentLayer(element.layer);
-			if (parentLayer instanceof ItemLayer) {
-				if (((ItemLayer) parentLayer).getItem() == item) {
-					elements.add(element);
+			if (checkParent) {
+				Layer parentLayer = _composition.getParentLayer(element.layer);
+				if (parentLayer instanceof ItemLayer<?>) {
+					if (((ItemLayer<?>) parentLayer).getItem() == item) {
+						elements.add(element);
+					}
 				}
 			}
 		}
