@@ -158,15 +158,17 @@ public class AnimatableIntegerElementDelegate extends AnimatableValueElementDele
 				}
 			});
 
-			composite.addPaintListener(new PaintListener() {
-				public void paintControl(PaintEvent e) {
-					Rectangle bounds = text.getBounds();
-					int height = composite.getBounds().height;
+			if (unit != null) {
+				composite.addPaintListener(new PaintListener() {
+					public void paintControl(PaintEvent e) {
+						Rectangle bounds = text.getBounds();
+						int height = composite.getBounds().height;
 
-					setValueForeground(e.gc, false);
-					drawString(e.gc, bounds.width + 5, 0, height, unit);
-				}
-			});
+						setValueForeground(e.gc, false);
+						drawString(e.gc, bounds.width + 5, 0, height, unit);
+					}
+				});
+			}
 
 			return composite;
 		}
