@@ -24,8 +24,10 @@
 package ch.kuramo.javie.app.views.layercomp;
 
 import java.lang.reflect.Method;
+import java.util.Collection;
 import java.util.List;
 import java.util.Map;
+import java.util.SortedMap;
 
 import org.eclipse.jface.viewers.CellEditor;
 import org.eclipse.jface.viewers.TextCellEditor;
@@ -42,13 +44,18 @@ import org.eclipse.swt.widgets.Event;
 import org.eclipse.swt.widgets.Listener;
 import org.eclipse.swt.widgets.TreeItem;
 
+import ch.kuramo.javie.api.Time;
+import ch.kuramo.javie.app.Activator;
 import ch.kuramo.javie.app.ImageUtil;
 import ch.kuramo.javie.app.project.ProjectManager;
 import ch.kuramo.javie.app.project.ProjectOperation;
+import ch.kuramo.javie.app.project.RemoveKeyframesOperation;
 import ch.kuramo.javie.app.views.LayerCompositionView;
 import ch.kuramo.javie.core.AnimatableValue;
 import ch.kuramo.javie.core.ArithmeticalAnimatableValue;
 import ch.kuramo.javie.core.JavieRuntimeException;
+import ch.kuramo.javie.core.Keyframe;
+import ch.kuramo.javie.core.LayerComposition;
 import ch.kuramo.javie.core.Util;
 
 public abstract class AnimatableValueElementDelegate<V> {
@@ -72,6 +79,8 @@ public abstract class AnimatableValueElementDelegate<V> {
 
 	protected final List<Rectangle> valueArea = Util.newList();
 
+	protected final SwitchGroup keyframeNavSwitch = new SwitchGroup();
+
 	protected int yKeyframe = Integer.MIN_VALUE;
 
 	protected V originalValue;
@@ -91,6 +100,10 @@ public abstract class AnimatableValueElementDelegate<V> {
 		this.element = element;
 		this.name = name;
 		this.avalue = avalue;
+
+		keyframeNavSwitch.add(new PrevKeyframeSwitch());
+		keyframeNavSwitch.add(new ToggleKeyframeSwitch());
+		keyframeNavSwitch.add(new NextKeyframeSwitch());
 	}
 
 	public Image getColumnImage(int columnIndex) {
@@ -153,6 +166,10 @@ public abstract class AnimatableValueElementDelegate<V> {
 				drawValue(gc, x, y, height, focused);
 				break;
 
+			case LayerCompositionView.SHOWHIDE_COL:
+				keyframeNavSwitch.paint(event);
+				break;
+
 			case LayerCompositionView.TIMELINE_COL:
 				TimelineManager tm = (TimelineManager) element.viewer.getData(LayerCompositionView.TIMELINE_MANAGER);
 				yKeyframe = tm.drawKeyframes(event, avalue);
@@ -202,6 +219,10 @@ public abstract class AnimatableValueElementDelegate<V> {
 				}
 				break;
 
+			case LayerCompositionView.SHOWHIDE_COL:
+				keyframeNavSwitch.mouseDown(event);
+				break;
+
 			case LayerCompositionView.TIMELINE_COL:
 				TimelineManager tm = (TimelineManager) element.viewer.getData(LayerCompositionView.TIMELINE_MANAGER);
 				tm.mouseDown(event, yKeyframe, element, avalue);
@@ -339,9 +360,18 @@ public abstract class AnimatableValueElementDelegate<V> {
 	}
 
 	protected int drawValue(
-			GC gc, int x, int y, int height, boolean focused, Double value, int precision, String unit) {
+			GC gc, int x, int y, int height, boolean focused,
+			Double value, int precision, String unit) {
 
-		x = drawValue(gc, x, y, height, focused, String.format("%." + precision + "f", value));
+		return drawValue(gc, x, y, height, focused, value, false, precision, unit);
+	}
+
+	protected int drawValue(
+			GC gc, int x, int y, int height, boolean focused,
+			Double value, boolean alwaysSign, int precision, String unit) {
+
+		x = drawValue(gc, x, y, height, focused, String.format(
+				(alwaysSign ? "%+." : "%.") + precision + "f", value));
 
 		if (unit != null) {
 			x = drawString(gc, x + 1, y, height, unit);
@@ -538,7 +568,14 @@ public abstract class AnimatableValueElementDelegate<V> {
 			if (columnIndex == LayerCompositionView.TIMELINE_COL) {
 				TextCellEditor editor = (TextCellEditor) viewer.getData(EDITOR_KEY);
 				if (editor == null) {
-					editor = new TextCellEditor(viewer.getTree(), SWT.SINGLE | SWT.BORDER);
+					editor = new TextCellEditor(viewer.getTree(), SWT.MULTI | SWT.V_SCROLL | SWT.BORDER) {
+						public LayoutData getLayoutData() {
+							LayoutData data = super.getLayoutData();
+							data.minimumHeight = 80;
+							data.verticalAlignment = SWT.TOP;
+							return data;
+						}
+					};
 					Control control = editor.getControl();
 					control.setBackground(control.getDisplay().getSystemColor(SWT.COLOR_WHITE));
 					viewer.setData(EDITOR_KEY, editor);
@@ -563,4 +600,165 @@ public abstract class AnimatableValueElementDelegate<V> {
 
 	}
 
+
+	private static final Image prevKeyframeImage			= Activator.getDefault().getImageRegistry().get(Activator.IMG_KFNAV_PREV);
+	private static final Image prevKeyframeDisabledImage	= Activator.getDefault().getImageRegistry().get(Activator.IMG_KFNAV_PREV_DISABLED);
+	private static final Image nextKeyframeImage			= Activator.getDefault().getImageRegistry().get(Activator.IMG_KFNAV_NEXT);
+	private static final Image nextKeyframeDisabledImage	= Activator.getDefault().getImageRegistry().get(Activator.IMG_KFNAV_NEXT_DISABLED);
+
+	private class PrevKeyframeSwitch implements Switch {
+
+		private final TimelineManager tm = (TimelineManager) element.viewer.getData(LayerCompositionView.TIMELINE_MANAGER);
+		private final LayerComposition comp = (LayerComposition) element.viewer.getData(LayerCompositionView.LAYER_COMPOSITION);
+
+		private Time getPrevTime() {
+			Time frameDuration = comp.getFrameDuration();
+			Time halfOfFrameDuration = new Time(frameDuration.timeValue/2, frameDuration.timeScale);
+
+			SortedMap<Time, Keyframe<V>> headMap = avalue.getKeyframeMap().headMap(
+					tm.getCurrentTime().subtract(halfOfFrameDuration));
+			if (headMap.isEmpty()) {
+				return null;
+			}
+
+			Time time = headMap.lastKey().add(halfOfFrameDuration);
+			return Time.fromFrameNumber(time.toFrameNumber(frameDuration), frameDuration);
+		}
+
+		public boolean hasBorder() {
+			return false;
+		}
+
+		public void mouseDown(MouseEvent event) {
+			if (avalue.hasKeyframe()) {
+				Time prevTime = getPrevTime();
+				if (prevTime != null) {
+					tm.setTime(prevTime);
+				}
+			}
+		}
+
+		public Image getImage() {
+			return avalue.hasKeyframe() ? (getPrevTime() != null) ? prevKeyframeImage : prevKeyframeDisabledImage : null;
+		}
+	}
+
+	private class NextKeyframeSwitch implements Switch {
+
+		private final TimelineManager tm = (TimelineManager) element.viewer.getData(LayerCompositionView.TIMELINE_MANAGER);
+		private final LayerComposition comp = (LayerComposition) element.viewer.getData(LayerCompositionView.LAYER_COMPOSITION);
+
+		private Time getNextTime() {
+			Time frameDuration = comp.getFrameDuration();
+			Time halfOfFrameDuration = new Time(frameDuration.timeValue/2, frameDuration.timeScale);
+
+			SortedMap<Time, Keyframe<V>> tailMap = avalue.getKeyframeMap().tailMap(
+					tm.getCurrentTime().add(frameDuration).subtract(halfOfFrameDuration));
+			if (tailMap.isEmpty()) {
+				return null;
+			}
+
+			Time time = tailMap.firstKey().add(halfOfFrameDuration);
+			return Time.fromFrameNumber(time.toFrameNumber(frameDuration), frameDuration);
+		}
+
+		public boolean hasBorder() {
+			return false;
+		}
+
+		public void mouseDown(MouseEvent event) {
+			if (avalue.hasKeyframe()) {
+				Time nextTime = getNextTime();
+				if (nextTime != null) {
+					tm.setTime(nextTime);
+				}
+			}
+		}
+
+		public Image getImage() {
+			return avalue.hasKeyframe() ? (getNextTime() != null) ? nextKeyframeImage : nextKeyframeDisabledImage : null;
+		}
+	}
+
+	private class ToggleKeyframeSwitch implements Switch {
+
+		private final TimelineManager tm = (TimelineManager) element.viewer.getData(LayerCompositionView.TIMELINE_MANAGER);
+		private final ProjectManager pm = (ProjectManager) element.viewer.getData(LayerCompositionView.PROJECT_MANAGER);
+		private final LayerComposition comp = (LayerComposition) element.viewer.getData(LayerCompositionView.LAYER_COMPOSITION);
+
+		private SortedMap<Time, Keyframe<V>> getKeyframeMap() {
+			Time frameDuration = comp.getFrameDuration();
+			Time halfOfFrameDuration = new Time(frameDuration.timeValue/2, frameDuration.timeScale);
+			Time time = tm.getCurrentTime();
+			Time t1 = time.subtract(halfOfFrameDuration);
+			Time t2 = time.add(frameDuration).subtract(halfOfFrameDuration);
+			return avalue.getKeyframeMap().subMap(t1, t2);
+		}
+
+		public boolean hasBorder() {
+			return avalue.hasKeyframe();
+		}
+
+		public void mouseDown(MouseEvent event) {
+			if (!avalue.hasKeyframe()) {
+				return;
+			}
+
+			Collection<Keyframe<V>> keyframes = getKeyframeMap().values();
+
+			if (keyframes.isEmpty()) {
+				pm.postOperation(element.createModifyKeyframeOperation(pm, tm.getCurrentTime(), value, null));
+
+			} else {
+				List<Object[]> data = Util.newList();
+
+				if (element instanceof LayerAnimatableValueElement) {
+					LayerAnimatableValueElement avalueElem = (LayerAnimatableValueElement) element;
+					String id = avalueElem.layer.getId();
+					String prop = avalueElem.property;
+
+					for (Keyframe<V> kf : keyframes) {
+						data.add(new Object[] { id, -1, prop, kf });
+					}
+
+				} else {
+					EffectAnimatableValueElement avalueElem = (EffectAnimatableValueElement) element;
+					String id = avalueElem.layer.getId();
+					int effectIndex = avalueElem.layer.getEffects().indexOf(avalueElem.effect);
+					String prop = avalueElem.descriptor.getName();
+
+					for (Keyframe<V> kf : keyframes) {
+						data.add(new Object[] { id, effectIndex, prop, kf });
+					}
+				}
+
+				pm.postOperation(new RemoveKeyframesOperation(pm, comp, data.toArray(new Object[data.size()][])));
+			}
+		}
+
+		public Image getImage() {
+			SortedMap<Time, Keyframe<V>> keyframeMap = getKeyframeMap();
+			if (keyframeMap.isEmpty()) {
+				return null;
+			}
+
+			Time time = tm.getCurrentTime();
+			Keyframe<V> kf = keyframeMap.get(time);
+			if (kf == null) {
+				SortedMap<Time, Keyframe<V>> head = keyframeMap.headMap(time);
+				SortedMap<Time, Keyframe<V>> tail = keyframeMap.tailMap(time);
+				Keyframe<V> kf1 = head.isEmpty() ? null : head.get(head.lastKey());
+				Keyframe<V> kf2 = tail.isEmpty() ? null : tail.get(tail.firstKey());
+				if (kf1 != null && kf2 != null) {
+					kf = kf2.time.subtract(time).before(time.subtract(kf1.time)) ? kf2 : kf1;
+				} else if (kf1 != null) {
+					kf = kf1;
+				} else if (kf2 != null) {
+					kf = kf2;
+				}
+			}
+			return ImageUtil.getKeyframeIcon(kf.interpolation, false);
+		}
+	}
+
 }
