@@ -78,8 +78,11 @@ public class LayerAnimatableValueElement extends AnimatableValueElement {
 			return new AnimatableVec2dElementDelegate(this, name, (AnimatableVec2d) avalue, unit);
 
 		} else if (avalue instanceof AnimatableDouble) {
-			return new AnimatableDoubleElementDelegate(this, name, (AnimatableDouble) avalue, unit);
-
+			if (property.startsWith("rotation")) {
+				return new AngleElementDelegate(this, name, (AnimatableDouble) avalue, unit);
+			} else {
+				return new AnimatableDoubleElementDelegate(this, name, (AnimatableDouble) avalue, unit);
+			}
 		} else if (avalue instanceof AnimatableInteger) {
 			return new AnimatableIntegerElementDelegate(this, name, (AnimatableInteger) avalue, unit);
 
