@@ -23,6 +23,7 @@
 
 package ch.kuramo.javie.app.views.layercomp;
 
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
@@ -43,6 +44,8 @@ import org.eclipse.swt.widgets.Event;
 import org.eclipse.ui.ISharedImages;
 import org.eclipse.ui.PlatformUI;
 
+import ch.kuramo.javie.api.BlendMode;
+import ch.kuramo.javie.app.EnumLabels;
 import ch.kuramo.javie.app.ImageUtil;
 import ch.kuramo.javie.app.project.ModifyLayerBlendModeOperation;
 import ch.kuramo.javie.app.project.ModifyLayerParentOperation;
@@ -52,7 +55,6 @@ import ch.kuramo.javie.app.project.RenameLayerOperation;
 import ch.kuramo.javie.app.views.ComboBoxViewerCellEditor;
 import ch.kuramo.javie.app.views.LayerCompositionView;
 import ch.kuramo.javie.app.widgets.WSWin32;
-import ch.kuramo.javie.core.BlendMode;
 import ch.kuramo.javie.core.CameraLayer;
 import ch.kuramo.javie.core.EffectableLayer;
 import ch.kuramo.javie.core.ItemLayer;
@@ -72,6 +74,15 @@ public class LayerElement extends Element {
 	private static final boolean COCOA = SWT.getPlatform().equals("cocoa");
 
 
+	private static final BlendMode[] layerBlendModes;
+
+	static{
+		List<BlendMode> list = Util.newList(Arrays.asList(BlendMode.values()));
+		list.remove(BlendMode.NONE);
+		layerBlendModes = list.toArray(new BlendMode[list.size()]);
+	}
+
+
 	public final Layer layer;
 
 	private final LayerComposition composition;
@@ -98,6 +109,16 @@ public class LayerElement extends Element {
 		this.layer = layer;
 		this.composition = (LayerComposition) viewer.getData(LayerCompositionView.LAYER_COMPOSITION);
 
+		if (WSWin32.isXPThemed()) {
+			showHideColumnSwitches.setMarginLeft(11);
+			switchesColumnSwitches.setMarginLeft(11);
+		}
+	}
+
+	public void createSwitches() {
+		showHideColumnSwitches.removeAll();
+		switchesColumnSwitches.removeAll();
+
 		ProjectManager pm = (ProjectManager) viewer.getData(LayerCompositionView.PROJECT_MANAGER);
 
 		if (LayerNature.isVideoNature(layer)) {
@@ -112,7 +133,6 @@ public class LayerElement extends Element {
 			showHideColumnSwitches.addPlaceholder();
 		}
 
-
 		switchesColumnSwitches.add(new ShySwitch(pm, layer));
 
 		if (LayerNature.isCTCRNature(layer)) {
@@ -144,19 +164,13 @@ public class LayerElement extends Element {
 		} else {
 			switchesColumnSwitches.addPlaceholder();
 		}
-
-
-		if (WSWin32.isXPThemed()) {
-			showHideColumnSwitches.setMarginLeft(11);
-			switchesColumnSwitches.setMarginLeft(11);
-		}
 	}
 
 	public Image getColumnImage(int columnIndex) {
 		switch (columnIndex) {
 			case LayerCompositionView.NAME_COL:
-				if (layer instanceof ItemLayer) {
-					return ImageUtil.getItemIcon(((ItemLayer) layer).getItem());
+				if (layer instanceof ItemLayer<?>) {
+					return ImageUtil.getItemIcon(((ItemLayer<?>) layer).getItem());
 				}
 				if (layer instanceof CameraLayer) {
 					return ImageUtil.getCameraIcon();
@@ -180,8 +194,8 @@ public class LayerElement extends Element {
 		switch (columnIndex) {
 			case LayerCompositionView.NAME_COL: {
 				String name;
-				if (layer instanceof ItemLayer) {
-					String itemName = ((ItemLayer) layer).getItem().getName();
+				if (layer instanceof ItemLayer<?>) {
+					String itemName = ((ItemLayer<?>) layer).getItem().getName();
 					if (isLayerNameMode()) {
 						name = layer.getName();
 						if (name.equals(itemName)) {
@@ -198,12 +212,12 @@ public class LayerElement extends Element {
 
 			case LayerCompositionView.MODE_COL:
 				return (layer instanceof MediaLayer)
-						? (COCOA ? "  " : "") + ((MediaLayer) layer).getBlendMode().label() : null;
+						? (COCOA ? "  " : "") + EnumLabels.getLabel(((MediaLayer) layer).getBlendMode()) : null;
 
 			case LayerCompositionView.TRACKMATTE_COL:
 				if (layer instanceof MediaLayer && getTrackMatteLayer() != null) {
 					TrackMatte trackMatte = ((MediaLayer) layer).getTrackMatte();
-					String label = trackMatte.label();
+					String label = EnumLabels.getLabel(trackMatte);
 					return (COCOA ? "  " : "") + (trackMatte != TrackMatte.NONE ? label : " - ");
 				}
 				return null;
@@ -213,10 +227,10 @@ public class LayerElement extends Element {
 				Layer parent = composition.getParentLayer(layer);
 				if (parent == null) {
 					parentName = " - ";
-				} else if (!(parent instanceof ItemLayer) || isLayerNameMode()) {
+				} else if (!(parent instanceof ItemLayer<?>) || isLayerNameMode()) {
 					parentName = parent.getName();
 				} else {
-					parentName = ((ItemLayer) parent).getItem().getName();
+					parentName = ((ItemLayer<?>) parent).getItem().getName();
 				}
 				return (COCOA ? "  " : "") + parentName;
 			}
@@ -416,7 +430,7 @@ public class LayerElement extends Element {
 						}
 					});
 				}
-				modeEditor.setInput(BlendMode.values());
+				modeEditor.setInput(layerBlendModes);
 				return modeEditor;
 
 			case LayerCompositionView.TRACKMATTE_COL:
@@ -483,8 +497,8 @@ public class LayerElement extends Element {
 			case LayerCompositionView.NAME_COL: {
 				String newName = (String) value;
 				if (newName.length() == 0) {
-					if (layer instanceof ItemLayer) {
-						newName = ((ItemLayer) layer).getItem().getName();
+					if (layer instanceof ItemLayer<?>) {
+						newName = ((ItemLayer<?>) layer).getItem().getName();
 					} else {
 						// TODO ItemLayer以外の場合、どうするのがいいか？　以下は暫定
 						if (layer instanceof TextLayer) newName = "テキスト";
@@ -512,7 +526,7 @@ public class LayerElement extends Element {
 
 	private static class BlendModeLabelProvider extends LabelProvider {
 		public String getText(Object element) {
-			return ((BlendMode) element).label();
+			return EnumLabels.getLabel((BlendMode) element);
 		}
 	}
 
@@ -520,16 +534,16 @@ public class LayerElement extends Element {
 		public String getText(Object element) {
 			TrackMatte trackMatte = (TrackMatte) element;
 			if (trackMatte == TrackMatte.NONE) {
-				return trackMatte.label();
+				return EnumLabels.getLabel(trackMatte);
 			} else {
 				String name;
 				MediaLayer trackMatteLayer = getTrackMatteLayer();
-				if (!(trackMatteLayer instanceof ItemLayer) || isLayerNameMode()) {
+				if (!(trackMatteLayer instanceof ItemLayer<?>) || isLayerNameMode()) {
 					name = trackMatteLayer.getName();
 				} else {
-					name = ((ItemLayer) trackMatteLayer).getItem().getName();
+					name = ((ItemLayer<?>) trackMatteLayer).getItem().getName();
 				}
-				return String.format("%s - %s", trackMatte.label(), name);
+				return String.format("%s - %s", EnumLabels.getLabel(trackMatte), name);
 			}
 		}
 	}
@@ -559,10 +573,10 @@ public class LayerElement extends Element {
 		public String getText(Object element) {
 			if (element == NO_PARENT) {
 				return "なし";
-			} else if (!(element instanceof ItemLayer) || isLayerNameMode()) {
+			} else if (!(element instanceof ItemLayer<?>) || isLayerNameMode()) {
 				return ((Layer) element).getName();
 			} else {
-				return ((ItemLayer) element).getItem().getName();
+				return ((ItemLayer<?>) element).getItem().getName();
 			}
 		}
 	}
