@@ -69,7 +69,7 @@ class TimeRemapElementDelegate extends AnimatableDoubleElementDelegate {
 		super(element, name, avalue);
 		this.layer = layer;
 
-		if (LayerNature.isVideoNature(layer)) {
+		if (LayerNature.isVideoNature(layer) && layer.getMediaInput().getVideoFrameDuration().timeValue > 0) {
 			frameDuration = layer.getMediaInput().getVideoFrameDuration();
 		} else {
 			LayerComposition comp = (LayerComposition) element.viewer.getData(LayerCompositionView.LAYER_COMPOSITION);
