@@ -113,12 +113,16 @@ public class TimelineManager {
 
 	private double zoom;
 
+	private Time newTime;
+
 	private Time currentTime;
 
 	private Time fromTime;
 
 	private Time toTime;
 
+	private boolean seeking;
+
 	private boolean wrapMode = true;
 
 	private final Map<Keyframe<?>, AnimatableValueElement> keyframeSelection = Util.newMap();
@@ -302,8 +306,17 @@ public class TimelineManager {
 		int scroll = scrollSlider.getSelection();
 
 		int left = timeToPixels(fromTime, frameDuration, zoom) - scroll;
-		int x = timeToPixels(currentTime, frameDuration, zoom) - scroll;
 
+//		if (seeking && newTime != null && !newTime.equals(currentTime)) {
+//			int x = timeToPixels(newTime, frameDuration, zoom) - scroll;
+//			if (left <= x) {
+//				x += timelineColumnX();
+//				e.gc.setForeground(e.display.getSystemColor(SWT.COLOR_DARK_RED));
+//				e.gc.drawLine(x, e.y, x, e.y+e.height);
+//			}
+//		}
+
+		int x = timeToPixels(currentTime, frameDuration, zoom) - scroll;
 		if (left <= x) {
 			x += timelineColumnX();
 			e.gc.setForeground(e.display.getSystemColor(SWT.COLOR_RED));
@@ -344,6 +357,12 @@ public class TimelineManager {
 			}
 		}
 
+		if (seeking && newTime != null && !newTime.equals(currentTime)) {
+			int x = timeToPixels(newTime, frameDuration, zoom) - scroll;
+			gc.setForeground(display.getSystemColor(SWT.COLOR_DARK_RED));
+			gc.drawLine(x, 0, x, h);
+		}
+
 		int x = timeToPixels(currentTime, frameDuration, zoom) - scroll;
 		gc.setForeground(display.getSystemColor(SWT.COLOR_RED));
 		gc.drawLine(x, 0, x, h);
@@ -352,6 +371,7 @@ public class TimelineManager {
 	private void mouseDownOnMeter(final MouseEvent e) {
 		final SortedSet<Time> snapTimes = collectSnapTimes(false, true, false, false);
 
+		seeking = true;
 		setWrapMode(false);
 		updateView(e.x, (e.stateMask & SWT.SHIFT) != 0, snapTimes);
 
@@ -392,6 +412,8 @@ public class TimelineManager {
 						display.removeFilter(SWT.MouseUp, this);
 						display.removeFilter(SWT.Deactivate, this);
 						setWrapMode(true);
+						seeking = false;
+						redraw();
 						break;
 				}
 			}
@@ -761,10 +783,15 @@ public class TimelineManager {
 
 	public void setTime(Time time) {
 		Time frameDuration = composition.getFrameDuration();
-		time = Time.fromFrameNumber(time.toFrameNumber(frameDuration), frameDuration);
+		newTime = Time.fromFrameNumber(time.toFrameNumber(frameDuration), frameDuration);
 
-		if (!time.equals(currentTime)) {
-			view.update(time);
+		if (!newTime.equals(currentTime)) {
+			view.update(newTime);
+
+			if (seeking) {
+				//redraw();
+				meter.redraw();
+			}
 		}
 	}
 
