@@ -44,10 +44,10 @@ public class AnimatableBoolean extends AbstractAnimatableValue<Boolean> implemen
 		return Boolean.valueOf((String) Context.jsToJava(jsValue, String.class));
 	}
 
-	public BooleanProperty createExpressionElement(final RenderContext renderContext) {
+	public BooleanProperty createExpressionElement(final CoreContext context) {
 		return new BooleanProperty() {
-			public boolean getValue()				{ return value(renderContext); }
-			public boolean valueAtTime(double t)	{ return AnimatableBoolean.this.valueAtTime(t, renderContext); }
+			public boolean getValue()				{ return value(context); }
+			public boolean valueAtTime(double t)	{ return AnimatableBoolean.this.valueAtTime(t, context); }
 		};
 	}
 
