@@ -23,9 +23,7 @@
 
 package ch.kuramo.javie.core;
 
-public interface MediaItemLayer extends MediaLayer, ItemLayer {
-
-	MediaItem getItem();
+public interface MediaItemLayer extends MediaLayer, ItemLayer<MediaItem> {
 
 	boolean isCTCR();
 
