@@ -23,6 +23,9 @@
 
 package ch.kuramo.javie.core;
 
+import ch.kuramo.javie.api.BlendMode;
+import ch.kuramo.javie.api.IAudioBuffer;
+
 public interface MediaLayer extends TransformableLayer, EffectableLayer {
 
 	boolean isVideoNature();
@@ -69,10 +72,10 @@ public interface MediaLayer extends TransformableLayer, EffectableLayer {
 
 
 	void setupVideoRenderer(
-			VideoLayerComposer composer, Camera camera, CollapseTransformation ct,
+			VideoLayerComposer composer, CollapseTransformation ct,
 			MotionBlur motionBlur, boolean frameBlendEnabled,
 			boolean withEffects, boolean withTransformation);
 
-	AudioBuffer renderAudioChunk(boolean withEffects, boolean withLevelsAndInOut);
+	IAudioBuffer renderAudioChunk(boolean withEffects, boolean withLevelsAndInOut);
 
 }
