@@ -23,10 +23,11 @@
 
 package ch.kuramo.javie.core;
 
+import ch.kuramo.javie.api.IVideoBuffer;
 import ch.kuramo.javie.api.Time;
 
 public interface VectorMediaInput extends MediaInput {
 
-	void rasterize(VideoBuffer resultBuffer, double[] mvMatrix, double[] prjMatrix, Time mediaTime);
+	void rasterize(IVideoBuffer resultBuffer, double[] mvMatrix, double[] prjMatrix, Time mediaTime);
 
 }
