@@ -23,11 +23,14 @@
 
 package ch.kuramo.javie.core;
 
+import ch.kuramo.javie.api.BlendMode;
+import ch.kuramo.javie.api.IVideoBuffer;
+
 public interface VideoLayerBuffer {
 
 	void dispose();
 
-	VideoBuffer getVideoBuffer();
+	IVideoBuffer getVideoBuffer();
 
 	DepthBuffer getDepthBuffer();
 
