@@ -26,9 +26,13 @@ package ch.kuramo.javie.core.internal;
 import java.util.Collections;
 import java.util.List;
 
+import net.arnx.jsonic.JSONHint;
 import ch.kuramo.javie.api.AudioMode;
+import ch.kuramo.javie.api.BlendMode;
 import ch.kuramo.javie.api.ColorMode;
+import ch.kuramo.javie.api.IAudioBuffer;
 import ch.kuramo.javie.api.IObjectArray;
+import ch.kuramo.javie.api.IVideoBuffer;
 import ch.kuramo.javie.api.Resolution;
 import ch.kuramo.javie.api.Size2i;
 import ch.kuramo.javie.api.Time;
@@ -40,8 +44,6 @@ import ch.kuramo.javie.core.AbstractTransformableLayer;
 import ch.kuramo.javie.core.AnimatableDouble;
 import ch.kuramo.javie.core.AnimatableString;
 import ch.kuramo.javie.core.AnimatableVec2d;
-import ch.kuramo.javie.core.AudioBuffer;
-import ch.kuramo.javie.core.BlendMode;
 import ch.kuramo.javie.core.Camera;
 import ch.kuramo.javie.core.CollapseTransformation;
 import ch.kuramo.javie.core.Composition;
@@ -60,7 +62,6 @@ import ch.kuramo.javie.core.ProjectDecodeException;
 import ch.kuramo.javie.core.TrackMatte;
 import ch.kuramo.javie.core.Util;
 import ch.kuramo.javie.core.VectorMediaInput;
-import ch.kuramo.javie.core.VideoBuffer;
 import ch.kuramo.javie.core.VideoLayerBuffer;
 import ch.kuramo.javie.core.VideoLayerComposer;
 import ch.kuramo.javie.core.VideoLayerRenderer;
@@ -76,14 +77,6 @@ import com.google.inject.Inject;
 
 public abstract class AbstractMediaLayer extends AbstractTransformableLayer implements MediaLayer {
 
-	private boolean _videoNature;
-
-	private boolean _audioNature;
-
-	private boolean _videoEnabled;
-
-	private boolean _audioEnabled;
-
 	private boolean _effectsEnabled = true;
 
 	private List<Effect> _effects = Util.newList();
@@ -126,59 +119,14 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 	private IArrayPools _arrayPools;
 
 
-	@Override
-	public void initialize() {
-		throw new UnsupportedOperationException("Use initialize(boolean, boolean) method instead.");
-	}
-
-	protected void initialize(boolean videoAvailable, boolean audioAvailable) {
-		if (!videoAvailable && !audioAvailable) {
-			throw new IllegalArgumentException();
-		}
-
-		super.initialize();
-
-		if (videoAvailable) {
-			_videoNature = true;
-			_videoEnabled = true;
-		}
-
-		if (audioAvailable) {
-			_audioNature = true;
-			_audioEnabled = true;
-		}
-	}
-
+	@JSONHint(ignore=true)
 	public boolean isVideoNature() {
-		return _videoNature;
-	}
-
-	public void setVideoNature(boolean videoNature) {
-		_videoNature = videoNature;
+		return getMediaInput().isVideoAvailable();
 	}
 
+	@JSONHint(ignore=true)
 	public boolean isAudioNature() {
-		return _audioNature;
-	}
-
-	public void setAudioNature(boolean audioNature) {
-		_audioNature = audioNature;
-	}
-
-	public boolean isVideoEnabled() {
-		return _videoEnabled;
-	}
-
-	public void setVideoEnabled(boolean enabled) {
-		_videoEnabled = enabled;
-	}
-
-	public boolean isAudioEnabled() {
-		return _audioEnabled;
-	}
-
-	public void setAudioEnabled(boolean enabled) {
-		_audioEnabled = enabled;
+		return getMediaInput().isAudioAvailable();
 	}
 
 	public boolean isEffectsEnabled() {
@@ -286,7 +234,7 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 	}
 
 	public void setupVideoRenderer(
-			VideoLayerComposer composer, Camera camera, CollapseTransformation ct,
+			VideoLayerComposer composer, CollapseTransformation ct,
 			MotionBlur motinBlur, boolean frameBlendEnabled,
 			boolean withEffects, boolean withTransformation) {
 
@@ -301,9 +249,9 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 		}
 
 		if (isVectorLayer() && LayerNature.isCTCR(this)) {
-			setupVideoRendererCR(composer, camera, ct, motinBlur, withEffects);
+			setupVideoRendererCR(composer, ct, motinBlur, withEffects);
 		} else {
-			setupVideoRendererNormal(composer, camera, ct, motinBlur, frameBlendEnabled, withEffects, withTransformation);
+			setupVideoRendererNormal(composer, ct, motinBlur, frameBlendEnabled, withEffects, withTransformation);
 		}
 	}
 
@@ -359,7 +307,7 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 	}
 
 	private void setupVideoRendererNormal(
-			VideoLayerComposer composer, final Camera camera, final CollapseTransformation ct,
+			VideoLayerComposer composer, final CollapseTransformation ct,
 			final MotionBlur motionBlur, final boolean frameBlendEnabled,
 			final boolean withEffects, final boolean withTransformation) {
 
@@ -382,7 +330,7 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 		if (withTransformation) {
 			mvMatrix = new double[16];
 			prjMatrix = new double[16];
-			createMatrix(camera, ct, offset, mvMatrix, prjMatrix);
+			createMatrix(ct, offset, mvMatrix, prjMatrix);
 		} else {
 			mvMatrix = null;
 			prjMatrix = null;
@@ -391,12 +339,14 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 
 		final Time time = _context.getTime();
 		final Composition comp = _context.getComposition();
+		final Camera camera = _context.getCamera();
 
 		VideoLayerRenderer r = new VideoLayerRenderer() {
 			public VideoLayerBuffer render(boolean withDepthBuffer) {
-				// コラップスしているときはここで時刻とコンポを再度設定する必要がある。
+				// コラップスしているときはここで時刻とコンポ、カメラを再度設定する必要がある。
 				_context.setTime(time);
 				_context.setComposition(comp);
+				_context.setCamera(camera);
 
 				final double opacity;
 				if (ct == null) {
@@ -405,12 +355,12 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 					opacity = _opacity.value(_context) / 100d * ct.getOpacity();
 				}
 
-				VideoBuffer effectedBuffer = null;
-				VideoBuffer trnsfrmdBuffer = null;
+				IVideoBuffer effectedBuffer = null;
+				IVideoBuffer trnsfrmdBuffer = null;
 				DepthBuffer depthBuffer = null;
 				try {
-					WrappedOperation<VideoBuffer> inputBufferOperation = new WrappedOperation<VideoBuffer>() {
-						public VideoBuffer execute() {
+					WrappedOperation<IVideoBuffer> inputBufferOperation = new WrappedOperation<IVideoBuffer>() {
+						public IVideoBuffer execute() {
 							if (frameBlendEnabled) {
 								FrameBlend frameBlend = LayerNature.getFrameBlend(AbstractMediaLayer.this);
 								if (frameBlend != FrameBlend.NONE) {
@@ -430,17 +380,16 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 								_vrSupport.fillDepth(depthBuffer, mvMatrix, prjMatrix);
 							}
 
-							final VideoBuffer mblurSrcBuf = effectedBuffer;
+							final IVideoBuffer mblurSrcBuf = effectedBuffer;
 							MotionBlurSampler sampler = new MotionBlurSampler() {
-								public void sample(double[] mvMatrix, double[] prjMatrix, VideoBuffer buffer) {
+								public void sample(double[] mvMatrix, double[] prjMatrix, IVideoBuffer buffer) {
 									_vrSupport.transform(mblurSrcBuf, buffer, null, mvMatrix, prjMatrix);
 								}
 							};
-							trnsfrmdBuffer = renderWithMotionBlur(camera, ct, motionBlur, offset, time, sampler);
+							trnsfrmdBuffer = renderWithMotionBlur(ct, motionBlur, offset, time, sampler);
 
 						} else {
 							trnsfrmdBuffer = _vrSupport.createVideoBuffer(_context.getColorMode(), camera.getViewportSize());
-							trnsfrmdBuffer.allocateAsTexture();
 							trnsfrmdBuffer.clear();
 
 							if (withDepthBuffer) {
@@ -493,7 +442,7 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 	}
 
 	private void setupVideoRendererCR(
-			VideoLayerComposer composer, final Camera camera, final CollapseTransformation ct,
+			VideoLayerComposer composer, final CollapseTransformation ct,
 			final MotionBlur motionBlur, final boolean withEffects) {
 
 		final VectorMediaInput input = (VectorMediaInput) getMediaInput();
@@ -503,17 +452,19 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 
 		final double[] mvMatrix = new double[16];
 		final double[] prjMatrix = new double[16];
-		createMatrix(camera, ct, null, mvMatrix, prjMatrix);
+		createMatrix(ct, null, mvMatrix, prjMatrix);
 
 
 		final Time time = _context.getTime();
 		final Composition comp = _context.getComposition();
+		final Camera camera = _context.getCamera();
 
 		VideoLayerRenderer r = new VideoLayerRenderer() {
 			public VideoLayerBuffer render(boolean withDepthBuffer) {
-				// コラップスしているときはここで時刻とコンポを再度設定する必要がある。
+				// コラップスしているときはここで時刻とコンポ、カメラを再度設定する必要がある。
 				_context.setTime(time);
 				_context.setComposition(comp);
+				_context.setCamera(camera);
 
 				final double opacity;
 				if (ct == null) {
@@ -522,8 +473,8 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 					opacity = _opacity.value(_context) / 100d * ct.getOpacity();
 				}
 
-				VideoBuffer effectedBuffer = null;
-				VideoBuffer resultBuffer = null;
+				IVideoBuffer effectedBuffer = null;
+				IVideoBuffer resultBuffer = null;
 				DepthBuffer depthBuffer = null;
 				try {
 					if (withDepthBuffer) {
@@ -537,27 +488,26 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 						}
 					};
 
-					WrappedOperation<VideoBuffer> inputBufferOperation = new WrappedOperation<VideoBuffer>() {
-						public VideoBuffer execute() {
-							VideoBuffer rasterizedBuffer = null;
+					WrappedOperation<IVideoBuffer> inputBufferOperation = new WrappedOperation<IVideoBuffer>() {
+						public IVideoBuffer execute() {
+							IVideoBuffer rasterizedBuffer = null;
 							try {
 								if (motionBlur != null && LayerNature.isMotionBlurEnabled(AbstractMediaLayer.this)) {
 									MotionBlurSampler sampler = new MotionBlurSampler() {
-										public void sample(double[] mvMatrix, double[] prjMatrix, VideoBuffer buffer) {
+										public void sample(double[] mvMatrix, double[] prjMatrix, IVideoBuffer buffer) {
 											input.rasterize(buffer, mvMatrix, prjMatrix, calcVideoMediaTime(input));
 										}
 									};
-									rasterizedBuffer = renderWithMotionBlur(camera, ct, motionBlur, null, _context.getTime(), sampler);
+									rasterizedBuffer = renderWithMotionBlur(ct, motionBlur, null, _context.getTime(), sampler);
 
 								} else {
 									rasterizedBuffer = _vrSupport.createVideoBuffer(_context.getColorMode(), camera.getViewportSize());
-									rasterizedBuffer.allocateAsTexture();
 									rasterizedBuffer.clear();
 
 									input.rasterize(rasterizedBuffer, mvMatrix, prjMatrix, calcVideoMediaTime(input));
 								}
 
-								VideoBuffer result = rasterizedBuffer;
+								IVideoBuffer result = rasterizedBuffer;
 								rasterizedBuffer = null;
 								return result;
 
@@ -578,7 +528,6 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 							effectedBuffer = null;
 						} else {
 							resultBuffer = _vrSupport.createVideoBuffer(_context.getColorMode(), resultBounds);
-							resultBuffer.allocateAsTexture();
 							resultBuffer.clear();
 							_vrSupport.copy(effectedBuffer, resultBuffer);
 						}
@@ -621,14 +570,13 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 	}
 
 	private void createMatrix(
-			final Camera camera, final CollapseTransformation ct, final Vec2d offset,
+			final CollapseTransformation ct, final Vec2d offset,
 			final double[] mvMatrix, final double[] prjMatrix) {
 
 		WrappedOperation<Object> operation = new WrappedOperation<Object>() {
 			public Object execute() {
-				if (camera == null) {
-					_vrSupport.resetMatrix();
-				} else if (LayerNature.isThreeD(AbstractMediaLayer.this)) {
+				Camera camera = _context.getCamera();
+				if (LayerNature.isThreeD(AbstractMediaLayer.this)) {
 					_vrSupport.setMatrix(camera.getModelView3D(), camera.getProjection3D());
 				} else {
 					_vrSupport.setMatrix(camera.getModelView2D(), camera.getProjection2D());
@@ -644,7 +592,7 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 		_vrSupport.pushMatrixAndExecute(operation);
 	}
 
-	private VideoBuffer blendFrames(FrameBlend frameBlend) {
+	private IVideoBuffer blendFrames(FrameBlend frameBlend) {
 		if (frameBlend == FrameBlend.NONE) throw new IllegalArgumentException();
 		if (frameBlend != FrameBlend.FRAME_MIX) throw new UnsupportedOperationException();
 
@@ -666,14 +614,13 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 		fpsRatio = Math.abs(fpsRatio);
 
 
-		List<VideoBuffer> buffers = Util.newList();
-		VideoBuffer buf32f = null;
+		List<IVideoBuffer> buffers = Util.newList();
+		IVideoBuffer buf32f = null;
 		try {
 			Resolution resolution = _context.getVideoResolution();
 			VideoBounds bounds = resolution.scale(input.getVideoFrameBounds());
 
 			buf32f = _vrSupport.createVideoBuffer(ColorMode.RGBA32_FLOAT, bounds);
-			buf32f.allocateAsTexture();
 			buf32f.clear();
 
 			List<Double> weights = Util.newList();
@@ -707,7 +654,7 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 
 					if (buffers.size() >= maxUnits) {
 						_vrSupport.accumulate(buffers, weights, buf32f);
-						for (VideoBuffer vb : buffers) vb.dispose();
+						for (IVideoBuffer vb : buffers) vb.dispose();
 						buffers.clear();
 						weights.clear();
 					}
@@ -743,36 +690,36 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 				_vrSupport.accumulate(buffers, weights, buf32f);
 			}
 
-			VideoBuffer resultBuffer;
+			IVideoBuffer resultBuffer;
 			if (_context.getColorMode() == ColorMode.RGBA32_FLOAT) {
 				resultBuffer = buf32f;
 				buf32f = null;
 			} else {
 				resultBuffer = _vrSupport.createVideoBuffer(_context.getColorMode(), buf32f.getBounds());
-				resultBuffer.allocateAsTexture();
 				_vrSupport.copy(buf32f, resultBuffer);
 			}
 			return resultBuffer;
 		} finally {
-			for (VideoBuffer vb : buffers) vb.dispose();
+			for (IVideoBuffer vb : buffers) vb.dispose();
 			if (buf32f != null) buf32f.dispose();
 		}
 	}
 
 	private interface MotionBlurSampler {
-		void sample(double[] mvMatrix, double[] prjMatrix, VideoBuffer buffer);
+		void sample(double[] mvMatrix, double[] prjMatrix, IVideoBuffer buffer);
 	}
 
-	private VideoBuffer renderWithMotionBlur(
-			Camera camera, CollapseTransformation ct, MotionBlur motionBlur,
+	private IVideoBuffer renderWithMotionBlur(
+			CollapseTransformation ct, MotionBlur motionBlur,
 			Vec2d offset, Time time, MotionBlurSampler sampler) {
 
-		List<VideoBuffer> buffers = Util.newList();
-		VideoBuffer buf = null;
-		VideoBuffer buf32f = null;
+		List<IVideoBuffer> buffers = Util.newList();
+		IVideoBuffer buf = null;
+		IVideoBuffer buf32f = null;
 		try {
-			buf32f = _vrSupport.createVideoBuffer(ColorMode.RGBA32_FLOAT, camera.getViewportSize());
-			buf32f.allocateAsTexture();
+			Size2i vpSize = _context.getCamera().getViewportSize();
+
+			buf32f = _vrSupport.createVideoBuffer(ColorMode.RGBA32_FLOAT, vpSize);
 			buf32f.clear();
 
 			double shutterAngle = motionBlur.getShutterAngle();
@@ -789,10 +736,9 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 				Time t = time.add(new Time((long)(frameDuration.timeValue*angle/360), frameDuration.timeScale));
 				_context.setTime(t);
 
-				createMatrix(camera, ct, offset, mvMatrix, prjMatrix);
+				createMatrix(ct, offset, mvMatrix, prjMatrix);
 
-				buf = _vrSupport.createVideoBuffer(_context.getColorMode(), camera.getViewportSize());
-				buf.allocateAsTexture();
+				buf = _vrSupport.createVideoBuffer(_context.getColorMode(), vpSize);
 				buf.clear();
 
 				sampler.sample(mvMatrix, prjMatrix, buf);
@@ -802,7 +748,7 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 
 				if (buffers.size() >= maxUnits) {
 					_vrSupport.accumulate(buffers, 1.0/samples, buf32f);
-					for (VideoBuffer vb : buffers) vb.dispose();
+					for (IVideoBuffer vb : buffers) vb.dispose();
 					buffers.clear();
 				}
 			}
@@ -810,27 +756,26 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 				_vrSupport.accumulate(buffers, 1.0/samples, buf32f);
 			}
 
-			VideoBuffer resultBuffer;
+			IVideoBuffer resultBuffer;
 			if (_context.getColorMode() == ColorMode.RGBA32_FLOAT) {
 				resultBuffer = buf32f;
 				buf32f = null;
 			} else {
 				resultBuffer = _vrSupport.createVideoBuffer(_context.getColorMode(), buf32f.getBounds());
-				resultBuffer.allocateAsTexture();
 				_vrSupport.copy(buf32f, resultBuffer);
 			}
 			return resultBuffer;
 
 		} finally {
-			for (VideoBuffer vb : buffers) vb.dispose();
+			for (IVideoBuffer vb : buffers) vb.dispose();
 			if (buf != null) buf.dispose();
 			if (buf32f != null) buf32f.dispose();
 		}
 	}
 
-	private AudioBuffer getTimeRemappedAudioChunk(MediaInput input) {
-		AudioBuffer ab1 = null;
-		AudioBuffer ab2 = null;
+	private IAudioBuffer getTimeRemappedAudioChunk(MediaInput input) {
+		IAudioBuffer ab1 = null;
+		IAudioBuffer ab2 = null;
 		IObjectArray<Double> times = null;
 
 		try {
@@ -900,7 +845,7 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 			}
 
 
-			AudioBuffer result = ab1;
+			IAudioBuffer result = ab1;
 			ab1 = null;
 			return result;
 
@@ -911,9 +856,9 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 		}
 	}
 
-	private AudioBuffer getRateChangedAudioChunk(MediaInput input, Time time, double rate) {
-		AudioBuffer ab1 = null;
-		AudioBuffer ab2 = null;
+	private IAudioBuffer getRateChangedAudioChunk(MediaInput input, Time time, double rate) {
+		IAudioBuffer ab1 = null;
+		IAudioBuffer ab2 = null;
 
 		try {
 			AudioMode audioMode = _context.getAudioMode();
@@ -985,7 +930,7 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 			}
 
 
-			AudioBuffer result = ab1;
+			IAudioBuffer result = ab1;
 			ab1 = null;
 			return result;
 
@@ -995,7 +940,7 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 		}
 	}
 
-	public AudioBuffer renderAudioChunk(boolean withEffects, boolean withLevelsAndInOut) {
+	public IAudioBuffer renderAudioChunk(boolean withEffects, boolean withLevelsAndInOut) {
 		// エフェクトからの参照時にこのチェックがあるとまずいので。同じチェックはコンポ側でもしている。
 		//if (!LayerNature.isAudioEnabled(this)) {
 		//	return null;
@@ -1006,8 +951,8 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 			return null;
 		}
 
-		WrappedOperation<AudioBuffer> inputOperation = new WrappedOperation<AudioBuffer>() {
-			public AudioBuffer execute() {
+		WrappedOperation<IAudioBuffer> inputOperation = new WrappedOperation<IAudioBuffer>() {
+			public IAudioBuffer execute() {
 				if (LayerNature.isTimeRemapEnabled(AbstractMediaLayer.this)) {
 					return getTimeRemappedAudioChunk(input);
 
@@ -1023,7 +968,7 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 			}
 		};
 
-		AudioBuffer ab = _aePipeline.doAudioEffects(
+		IAudioBuffer ab = _aePipeline.doAudioEffects(
 				(withEffects && _effectsEnabled) ? _effects : Collections.<Effect>emptyList(), inputOperation);
 
 		if (withLevelsAndInOut) {
