@@ -31,7 +31,6 @@ import ch.kuramo.javie.api.IObjectArray;
 import ch.kuramo.javie.api.services.IArrayPools;
 import ch.kuramo.javie.api.services.IAudioEffectContext;
 import ch.kuramo.javie.core.AnimatableValue;
-import ch.kuramo.javie.core.AudioBuffer;
 import ch.kuramo.javie.core.AudioEffect;
 import ch.kuramo.javie.core.Effect;
 import ch.kuramo.javie.core.Util;
@@ -58,11 +57,11 @@ public class AudioEffectPipelineImpl implements AudioEffectPipeline {
 	private IArrayPools _arrayPools;
 
 
-	public AudioBuffer doAudioEffects(List<Effect> effects, WrappedOperation<AudioBuffer> inputOperation) {
+	public IAudioBuffer doAudioEffects(List<Effect> effects, WrappedOperation<IAudioBuffer> inputOperation) {
 		AudioEffectContextImpl impl = new AudioEffectContextImpl(enabledAudioEffectsOnly(effects), inputOperation);
 		AudioEffectContextProxy.push(impl);
 		try {
-			return (AudioBuffer) impl.doPreviousEffect();
+			return impl.doPreviousEffect();
 		} finally {
 			AudioEffectContextProxy.pop();
 		}
@@ -84,10 +83,12 @@ public class AudioEffectPipelineImpl implements AudioEffectPipeline {
 
 		private List<AudioEffect> effects;
 
-		private final WrappedOperation<AudioBuffer> inputOperation;
+		private final WrappedOperation<IAudioBuffer> inputOperation;
+
+		private AudioEffect currentEffect;
 
 
-		private AudioEffectContextImpl(List<AudioEffect> effects, WrappedOperation<AudioBuffer> inputOperation) {
+		private AudioEffectContextImpl(List<AudioEffect> effects, WrappedOperation<IAudioBuffer> inputOperation) {
 			super(_context, _vrSupport, _arSupport);
 
 			this.effects = effects;
@@ -111,34 +112,46 @@ public class AudioEffectPipelineImpl implements AudioEffectPipeline {
 
 		public IAudioBuffer doPreviousEffect() {
 			if (effects.isEmpty()) {
-				AudioBuffer ab = _context.saveAndExecute(inputOperation);
+				IAudioBuffer ab = _context.saveAndExecute(inputOperation);
 				if (ab == null) {
 					ab = _arSupport.createAudioBuffer();
 					_arSupport.clear(ab);
 				}
-				return (IAudioBuffer) ab;
+				return ab;
 			}
 
-			final AudioEffect prevEffect = effects.get(effects.size() - 1);
-
 			return executePrevious(new WrappedOperation<IAudioBuffer>() {
 				public IAudioBuffer execute() {
-					IAudioBuffer buffer = prevEffect.doAudioEffect();
+					IAudioBuffer buffer = currentEffect.doAudioEffect();
 					return (buffer != null) ? buffer : doPreviousEffect();
 				}
 			});
 		}
 
 		private <T> T executePrevious(final WrappedOperation<T> wop) {
+			AudioEffect savedCurrent = currentEffect;
 			List<AudioEffect> savedEffects = effects;
 			try {
-				effects = effects.subList(0, effects.size() - 1);
+
+				int lastIndex = effects.size() - 1;
+				currentEffect = effects.get(lastIndex);
+				effects = effects.subList(0, lastIndex);
+
 				return _context.saveAndExecute(wop);
+
 			} finally {
 				effects = savedEffects;
+				currentEffect = savedCurrent;
 			}
 		}
 
+		public String getEffectName() {
+			if (currentEffect == null) {
+				throw new IllegalStateException("no current Effect");
+			}
+			return currentEffect.getName();
+		}
+
 	}
 
 }
