@@ -45,6 +45,7 @@ import ch.kuramo.javie.core.Composition;
 import ch.kuramo.javie.core.CompositionItem;
 import ch.kuramo.javie.core.FileItem;
 import ch.kuramo.javie.core.Folder;
+import ch.kuramo.javie.core.ImageSequenceItem;
 import ch.kuramo.javie.core.JavieRuntimeException;
 import ch.kuramo.javie.core.LayerComposition;
 import ch.kuramo.javie.core.MediaInput;
@@ -59,6 +60,7 @@ import ch.kuramo.javie.core.annotations.ProjectElement;
 import ch.kuramo.javie.core.internal.CameraLayerImpl;
 import ch.kuramo.javie.core.internal.CompositionItemImpl;
 import ch.kuramo.javie.core.internal.FolderImpl;
+import ch.kuramo.javie.core.internal.ImageSequenceItemImpl;
 import ch.kuramo.javie.core.internal.LayerCompositionImpl;
 import ch.kuramo.javie.core.internal.MediaFileItem;
 import ch.kuramo.javie.core.internal.MediaItemLayerImpl;
@@ -154,11 +156,25 @@ public class ProjectElementFactoryImpl implements ProjectElementFactory {
 		return folder;
 	}
 
+	public ImageSequenceItem newImageSequenceItem(File file, Time frameDuration) {
+		ImageSequenceItemImpl item = _injector.getInstance(ImageSequenceItemImpl.class);
+		item.initialize(file, frameDuration);
+		if (item.getMediaInput() != null) {
+			return item;
+		} else {
+			item.dispose();
+			return null;
+		}
+	}
+
 	public FileItem newFileItem(File file) {
 		MediaFileItem item = _injector.getInstance(MediaFileItem.class);
 		item.initialize(file);
 		if (item.getMediaInput() != null) {
 			return item;
+		} else {
+			item.dispose();
+			item = null;
 		}
 
 		// 他にFileItemの実装を作ったら、ここに入れる。
