@@ -25,12 +25,14 @@ package ch.kuramo.javie.core.internal.services;
 
 import java.util.List;
 
+import ch.kuramo.javie.api.ICamera;
 import ch.kuramo.javie.api.IVideoBuffer;
 import ch.kuramo.javie.api.VideoBounds;
+import ch.kuramo.javie.api.IVideoBuffer.TextureFilter;
+import ch.kuramo.javie.api.IVideoBuffer.TextureWrapMode;
 import ch.kuramo.javie.api.services.IVideoEffectContext;
 import ch.kuramo.javie.core.Effect;
 import ch.kuramo.javie.core.Util;
-import ch.kuramo.javie.core.VideoBuffer;
 import ch.kuramo.javie.core.VideoEffect;
 import ch.kuramo.javie.core.WrappedOperation;
 import ch.kuramo.javie.core.services.AudioRenderSupport;
@@ -65,15 +67,15 @@ public class VideoEffectPipelineImpl implements VideoEffectPipeline {
 		}
 	}
 
-	public VideoBuffer doVideoEffects(List<Effect> effects,
+	public IVideoBuffer doVideoEffects(List<Effect> effects,
 			WrappedOperation<VideoBounds> inputBoundsOperation,
-			WrappedOperation<VideoBuffer> inputBufferOperation) {
+			WrappedOperation<IVideoBuffer> inputBufferOperation) {
 
 		VideoEffectContextImpl impl = new VideoEffectContextImpl(
 				enabledVideoEffectsOnly(effects), inputBoundsOperation, inputBufferOperation);
 		VideoEffectContextProxy.push(impl);
 		try {
-			return (VideoBuffer) impl.doPreviousEffect();
+			return impl.doPreviousEffect();
 		} finally {
 			VideoEffectContextProxy.pop();
 		}
@@ -97,7 +99,9 @@ public class VideoEffectPipelineImpl implements VideoEffectPipeline {
 
 		private final WrappedOperation<VideoBounds> inputBoundsOperation;
 
-		private final WrappedOperation<VideoBuffer> inputBufferOperation;
+		private final WrappedOperation<IVideoBuffer> inputBufferOperation;
+
+		private VideoEffect currentEffect;
 
 		private boolean gettingBounds;
 
@@ -105,7 +109,7 @@ public class VideoEffectPipelineImpl implements VideoEffectPipeline {
 		private VideoEffectContextImpl(
 				List<VideoEffect> effects,
 				WrappedOperation<VideoBounds> inputBoundsOperation,
-				WrappedOperation<VideoBuffer> inputBufferOperation) {
+				WrappedOperation<IVideoBuffer> inputBufferOperation) {
 
 			super(_context, _vrSupport, _arSupport);
 
@@ -127,11 +131,9 @@ public class VideoEffectPipelineImpl implements VideoEffectPipeline {
 					return _context.saveAndExecute(inputBoundsOperation);
 				}
 
-				final VideoEffect prevEffect = effects.get(effects.size() - 1);
-
 				return executePrevious(new WrappedOperation<VideoBounds>() {
 					public VideoBounds execute() {
-						VideoBounds bounds = prevEffect.getVideoBounds();
+						VideoBounds bounds = currentEffect.getVideoBounds();
 						return (bounds != null) ? bounds : getPreviousBounds();
 					}
 				});
@@ -146,24 +148,43 @@ public class VideoEffectPipelineImpl implements VideoEffectPipeline {
 			}
 
 			if (effects.isEmpty()) {
-				return (IVideoBuffer) _context.saveAndExecute(inputBufferOperation);
+				return _context.saveAndExecute(inputBufferOperation);
 			}
 
-			final VideoEffect prevEffect = effects.get(effects.size() - 1);
-
 			return executePrevious(new WrappedOperation<IVideoBuffer>() {
 				public IVideoBuffer execute() {
-					IVideoBuffer buffer = prevEffect.doVideoEffect();
-					return (buffer != null) ? buffer : doPreviousEffect();
+					IVideoBuffer buffer = currentEffect.doVideoEffect();
+					if (buffer == null) {
+						return doPreviousEffect();
+					}
+					// TODO エフェクトから返されたバッファのColorModeがコンテキストのColorModeと異なる場合、
+					//      コンテキストのColorModeのバッファにコピーして返す。また、警告をログに出す。
+					//      ただし、このコメントを書いた時点ではコンテキストのColorModeと異なるバッファを
+					//      エフェクト内で作成することはできない。
+					//if (buffer.getColorMode() != _context.getColorMode()) {
+					//	IVideoBuffer copy = _vrSupport.createVideoBuffer(_context.getColorMode(), buffer.getBounds());
+					//	_vrSupport.copy(buffer, copy);
+					//	buffer.dispose();
+					//	return copy;
+					//}
+					if (buffer.isTexture()) {
+						// TODO テクスチャの現在の補間モードとラッピングモードが NEAREST / CLAMP_TO_BORDER 以外の場合、警告をログに出す。
+						buffer.setTextureFilter(TextureFilter.NEAREST);
+						buffer.setTextureWrapMode(TextureWrapMode.CLAMP_TO_BORDER);
+					}
+					return buffer;
 				}
 			});
 		}
 
 		private <T> T executePrevious(final WrappedOperation<T> wop) {
+			VideoEffect savedCurrent = currentEffect;
 			List<VideoEffect> savedEffects = effects;
 			try {
 
-				effects = effects.subList(0, effects.size() - 1);
+				int lastIndex = effects.size() - 1;
+				currentEffect = effects.get(lastIndex);
+				effects = effects.subList(0, lastIndex);
 
 				return _context.saveAndExecute(new WrappedOperation<T>() {
 					public T execute() {
@@ -173,9 +194,20 @@ public class VideoEffectPipelineImpl implements VideoEffectPipeline {
 
 			} finally {
 				effects = savedEffects;
+				currentEffect = savedCurrent;
 			}
 		}
 
+		public ICamera getCamera() {
+			return _context.getCamera();
+		}
+
+		public String getEffectName() {
+			if (currentEffect == null) {
+				throw new IllegalStateException("no current Effect");
+			}
+			return currentEffect.getName();
+		}
 
 	}
 
