@@ -28,29 +28,36 @@ import org.mozilla.javascript.ContextFactory;
 import ch.kuramo.javie.api.services.IAntiAliasSupport;
 import ch.kuramo.javie.api.services.IArrayPools;
 import ch.kuramo.javie.api.services.IAudioEffectContext;
+import ch.kuramo.javie.api.services.IBlendSupport;
+import ch.kuramo.javie.api.services.IBlurSupport;
+import ch.kuramo.javie.api.services.IConvolutionSupport;
 import ch.kuramo.javie.api.services.IShaderRegistry;
 import ch.kuramo.javie.api.services.IVideoEffectContext;
+import ch.kuramo.javie.api.services.IVideoRenderSupport;
 import ch.kuramo.javie.core.internal.services.AnimatableValueCollectorImpl;
 import ch.kuramo.javie.core.internal.services.AntiAliasSupportProxy;
 import ch.kuramo.javie.core.internal.services.ArrayPoolsImpl;
 import ch.kuramo.javie.core.internal.services.AudioEffectContextProxy;
 import ch.kuramo.javie.core.internal.services.AudioEffectPipelineImpl;
 import ch.kuramo.javie.core.internal.services.AudioRenderSupportImpl;
+import ch.kuramo.javie.core.internal.services.BlendSupportImpl;
+import ch.kuramo.javie.core.internal.services.BlurSupportImpl;
+import ch.kuramo.javie.core.internal.services.ConvolutionSupportImpl;
 import ch.kuramo.javie.core.internal.services.EffectRegistryImpl;
 import ch.kuramo.javie.core.internal.services.FontListImpl;
 import ch.kuramo.javie.core.internal.services.FontManagerProxy;
 import ch.kuramo.javie.core.internal.services.GLGlobalImpl;
-import ch.kuramo.javie.core.internal.services.MediaFileInputFactoryImpl;
+import ch.kuramo.javie.core.internal.services.MediaSourceFactoryImpl;
 import ch.kuramo.javie.core.internal.services.ProjectDecoderImpl;
 import ch.kuramo.javie.core.internal.services.ProjectElementFactoryImpl;
 import ch.kuramo.javie.core.internal.services.ProjectEncoderImpl;
-import ch.kuramo.javie.core.internal.services.RenderBufferPoolImpl;
 import ch.kuramo.javie.core.internal.services.RenderContextProxy;
 import ch.kuramo.javie.core.internal.services.ShaderRegistryImpl;
 import ch.kuramo.javie.core.internal.services.SynchronousTaskThreadImpl;
 import ch.kuramo.javie.core.internal.services.VideoEffectContextProxy;
 import ch.kuramo.javie.core.internal.services.VideoEffectPipelineImpl;
 import ch.kuramo.javie.core.internal.services.VideoRenderSupportImpl;
+import ch.kuramo.javie.core.internal.services.VideoRenderSupportImplOld;
 
 import com.google.inject.AbstractModule;
 import com.google.inject.Scopes;
@@ -74,8 +81,8 @@ public class JavieCoreModule extends AbstractModule {
 				.to(ProjectDecoderImpl.class)
 				.in(Scopes.SINGLETON);
 
-		bind(MediaFileInputFactory.class)
-				.to(MediaFileInputFactoryImpl.class)
+		bind(MediaSourceFactory.class)
+				.to(MediaSourceFactoryImpl.class)
 				.in(Scopes.SINGLETON);
 
 		bind(GLGlobal.class)
@@ -99,6 +106,10 @@ public class JavieCoreModule extends AbstractModule {
 				.in(Scopes.SINGLETON);
 
 		bind(VideoRenderSupport.class)
+				.to(VideoRenderSupportImplOld.class)
+				.in(Scopes.SINGLETON);
+
+		bind(IVideoRenderSupport.class)
 				.to(VideoRenderSupportImpl.class)
 				.in(Scopes.SINGLETON);
 
@@ -122,14 +133,18 @@ public class JavieCoreModule extends AbstractModule {
 				.to(AudioEffectContextProxy.class)
 				.in(Scopes.SINGLETON);
 
-		bind(RenderBufferPool.class)
-				.to(RenderBufferPoolImpl.class)
-				.in(Scopes.SINGLETON);
-
 		bind(IAntiAliasSupport.class)
 				.to(AntiAliasSupportProxy.class)
 				.in(Scopes.SINGLETON);
 
+		bind(IConvolutionSupport.class)
+				.to(ConvolutionSupportImpl.class)
+				.in(Scopes.SINGLETON);
+
+		bind(IBlurSupport.class)
+				.to(BlurSupportImpl.class)
+				.in(Scopes.SINGLETON);
+
 		bind(FontList.class)
 				.to(FontListImpl.class)
 				.in(Scopes.SINGLETON);
@@ -153,6 +168,9 @@ public class JavieCoreModule extends AbstractModule {
 
 		bind(AnimatableValueCollector.class)
 				.to(AnimatableValueCollectorImpl.class);
+
+		bind(IBlendSupport.class)
+				.to(BlendSupportImpl.class);
 	}
 
 }
