@@ -36,6 +36,7 @@ import ch.kuramo.javie.core.Composition;
 import ch.kuramo.javie.core.CompositionItem;
 import ch.kuramo.javie.core.FileItem;
 import ch.kuramo.javie.core.Folder;
+import ch.kuramo.javie.core.ImageSequenceItem;
 import ch.kuramo.javie.core.LayerComposition;
 import ch.kuramo.javie.core.MediaItem;
 import ch.kuramo.javie.core.MediaItemLayer;
@@ -57,6 +58,8 @@ public interface ProjectElementFactory {
 
 	FileItem newFileItem(File file);
 
+	ImageSequenceItem newImageSequenceItem(File file, Time frameDuration);
+
 	SolidColorItem newSolidColorItem(Color color, Size2i size);
 
 	CompositionItem newCompositionItem(Composition composition);
