@@ -56,16 +56,21 @@
 }
 
 -(id) initWithFile:(NSString*)filename
-		cglContext:(CGLContextObj)cglContext
-	cglPixelFormat:(CGLPixelFormatObj)cglPixelFormat;
+				  :(CGLContextObj)cglContext
+				  :(CGLPixelFormatObj)cglPixelFormat;
 
 -(long) width;
 -(long) height;
 
 -(OSStatus) frameImageAtTime:(QTTime)time
-					 texture:(GLint)texture
-					   width:(GLint)width
-					  height:(GLint)height;
+							:(GLint)texture
+							:(GLint)width
+							:(GLint)height
+							:(int)alphaType
+							:(float)colorMatteR
+							:(float)colorMatteG
+							:(float)colorMatteB
+							:(BOOL)flipVertical;
 
 @end
 
@@ -82,8 +87,8 @@
 -(OSStatus) setAudioDescription:(NSDictionary*)description;
 
 -(OSStatus) audioChunkFromTime:(QTTime)time
-						buffer:(void*)buffer
-						offset:(long)offset
-					frameCount:(long)frameCount;
+							  :(void*)buffer
+							  :(long)offset
+							  :(long)frameCount;
 
 @end
