@@ -40,10 +40,14 @@ public class KeyingShaders {
 		"void main(void)",
 		"{",
 		"	vec4 color = texture2DRect(texture, gl_TexCoord[0].st);",
-		"	vec3 rgb = color.rgb/color.a;",		// color.a == 0.0 の場合でも残りの計算に影響はない。
-		"	color.a = min(color.a, color.a*(threshold-rgb.b*(1.0-rgb.r))/t_minus_c);",
-		"	rgb.b = min(rgb.b, rgb.g);",
-		"	gl_FragColor = vec4(rgb, 1.0)*color.a;",
+		"	if (color.a != 0.0) {",
+		"		vec3 rgb = color.rgb/color.a;",
+		"		color.a = clamp(color.a*(threshold-rgb.b*(1.0-rgb.r))/t_minus_c, 0.0, color.a);",
+		"		rgb.b = min(rgb.b, rgb.g);",
+		"		gl_FragColor = vec4(rgb, 1.0)*color.a;",
+		"	} else {",
+		"		gl_FragColor = vec4(0.0);",
+		"	}",
 		"}"
 	};
 
@@ -56,9 +60,13 @@ public class KeyingShaders {
 		"void main(void)",
 		"{",
 		"	vec4 color = texture2DRect(texture, gl_TexCoord[0].st);",
-		"	vec3 rgb = color.rgb/color.a;",
-		"	float a = min(color.a, color.a*(threshold-rgb.b*(1.0-rgb.r))/t_minus_c);",
-		"	gl_FragColor = vec4(a, a, a, 1.0);",
+		"	if (color.a != 0.0) {",
+		"		vec3 rgb = color.rgb/color.a;",
+		"		float a = clamp(color.a*(threshold-rgb.b*(1.0-rgb.r))/t_minus_c, 0.0, color.a);",
+		"		gl_FragColor = vec4(a, a, a, 1.0);",
+		"	} else {",
+		"		gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);",
+		"	}",
 		"}"
 	};
 
@@ -71,10 +79,14 @@ public class KeyingShaders {
 		"void main(void)",
 		"{",
 		"	vec4 color = texture2DRect(texture, gl_TexCoord[0].st);",
-		"	vec3 rgb = color.rgb/color.a;",
-		"	color.a = min(color.a, color.a*(threshold-rgb.g*(1.0-rgb.r))/t_minus_c);",
-		"	rgb.g = min(rgb.g, rgb.r);",
-		"	gl_FragColor = vec4(rgb, 1.0)*color.a;",
+		"	if (color.a != 0.0) {",
+		"		vec3 rgb = color.rgb/color.a;",
+		"		color.a = clamp(color.a*(threshold-rgb.g*(1.0-rgb.r))/t_minus_c, 0.0, color.a);",
+		"		rgb.g = min(rgb.g, rgb.r);",
+		"		gl_FragColor = vec4(rgb, 1.0)*color.a;",
+		"	} else {",
+		"		gl_FragColor = vec4(0.0);",
+		"	}",
 		"}"
 	};
 
@@ -87,9 +99,13 @@ public class KeyingShaders {
 		"void main(void)",
 		"{",
 		"	vec4 color = texture2DRect(texture, gl_TexCoord[0].st);",
-		"	vec3 rgb = color.rgb/color.a;",
-		"	float a = min(color.a, color.a*(threshold-rgb.g*(1.0-rgb.r))/t_minus_c);",
-		"	gl_FragColor = vec4(a, a, a, 1.0);",
+		"	if (color.a != 0.0) {",
+		"		vec3 rgb = color.rgb/color.a;",
+		"		float a = clamp(color.a*(threshold-rgb.g*(1.0-rgb.r))/t_minus_c, 0.0, color.a);",
+		"		gl_FragColor = vec4(a, a, a, 1.0);",
+		"	} else {",
+		"		gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);",
+		"	}",
 		"}"
 	};
 
@@ -173,14 +189,22 @@ public class KeyingShaders {
 				"void main(void)",
 				"{",
 				"	vec4 color = texture2DRect(texture, gl_TexCoord[0].st);",
-				"	vec3 rgb = color.rgb/color.a;",
+				"	float a1, a2;",
 				"",
-				"	vec2 uv = vec2(dot(uVec, rgb), dot(vVec, rgb));",
-				"	float d = distance(keyUV, uv);",
-				"	float a1 = (d-similarity)/blend;",
+				"	if (color.a != 0.0) {",
+				"		vec3 rgb = color.rgb/color.a;",
 				"",
-				"	float y = dot(yVec, rgb);",
-				"	float a2 = clamp(1.0-(y-cutoff)/t_minus_c, 0.0, 1.0);",
+				"		vec2 uv = vec2(dot(uVec, rgb), dot(vVec, rgb));",
+				"		float d = distance(keyUV, uv);",
+				"		a1 = (d-similarity)/blend;",
+				"",
+				"		float y = dot(yVec, rgb);",
+				"		a2 = clamp(1.0-(y-cutoff)/t_minus_c, 0.0, 1.0);",
+				"",
+				"	} else {",
+				"		a1 = 0.0;",
+				"		a2 = 0.0;",
+				"	}",
 				"",
 				"	if (a1 > 0.0) {",
 				"		float a = min(1.0, a1)*(1.0-a2)+a2;",
@@ -320,49 +344,56 @@ public class KeyingShaders {
 				"void main(void)",
 				"{",
 				"	vec4 color = texture2DRect(texture, gl_TexCoord[0].st);",
-				"	vec3 unmult = color.rgb/color.a;",
-				"	vec3 hsl = rgb2hsl(unmult);",
-				"	vec3 bleachDelta = hsl-bleachHSL;",
-				"	hsl.y *= 1.0-abs(hsl.z-0.5)*2.0;",
+				"	if (color.a != 0.0) {",
+				"		vec3 unmult = color.rgb/color.a;",
+				"		vec3 hsl = rgb2hsl(unmult);",
+				"		vec3 bleachDelta = hsl-bleachHSL;",
+				"		hsl.y *= 1.0-abs(hsl.z-0.5)*2.0;",
 				"",
-				"	vec3 a3 = max((similarityMin-hsl)/blendMin, (hsl-similarityMax)/blendMax);",
-				"	a3.x = min(a3.x, (similarityMin.x-(hsl.x-1.0))/blendMin.x);",
-				"	a3.x = min(a3.x, ((hsl.x+1.0)-similarityMax.x)/blendMax.x);",
+				"		vec3 a3 = max((similarityMin-hsl)/blendMin, (hsl-similarityMax)/blendMax);",
+				"		a3.x = min(a3.x, (similarityMin.x-(hsl.x-1.0))/blendMin.x);",
+				"		a3.x = min(a3.x, ((hsl.x+1.0)-similarityMax.x)/blendMax.x);",
 				"",
-				"	a3 = vec3_111 - clamp(a3, 0.0, 1.0);",
-				"	a3 = 0.5*(1.0-cos(PI*a3));",
-				"	float a = 1.0 - a3.x*a3.y*a3.z;",
+				"		a3 = vec3_111 - clamp(a3, 0.0, 1.0);",
+				"		a3 = 0.5*(1.0-cos(PI*a3));",
+				"		float a = 1.0 - a3.x*a3.y*a3.z;",
 				"",
-				"	if (similarityMin.y-blendMin.y < 0.0 && hsl.y < blendMin.y-similarityMin.y) {",
-				"		float y = 1.0 - clamp((hsl.y+similarityMin.y)/blendMin.y, 0.0, 1.0);",
-				"		y = 0.5*(1.0-cos(PI*y));",
-				"		a = min(a, 1.0 - y*a3.z);",
-				"	}",
+				"		if (similarityMin.y-blendMin.y < 0.0 && hsl.y < blendMin.y-similarityMin.y) {",
+				"			float y = 1.0 - clamp((hsl.y+similarityMin.y)/blendMin.y, 0.0, 1.0);",
+				"			y = 0.5*(1.0-cos(PI*y));",
+				"			a = min(a, 1.0 - y*a3.z);",
+				"		}",
 				"",
-				"	float bleachLevel = (bleachDelta.x-floor(bleachDelta.x))*6.0;",
-				"	if (bleachLevel > 3.0) {",
-				"		bleachLevel = (6.0-bleachLevel)/correctionHue.x;",
-				"	} else {",
-				"		bleachLevel /= correctionHue.y;",
-				"	}",
+				"		float bleachLevel = (bleachDelta.x-floor(bleachDelta.x))*6.0;",
+				"		if (bleachLevel > 3.0) {",
+				"			bleachLevel = (6.0-bleachLevel)/correctionHue.x;",
+				"		} else {",
+				"			bleachLevel /= correctionHue.y;",
+				"		}",
 				"",
-				"	vec3 b3 = vec3_111 - vec3(min(bleachLevel, 1.0), min(abs(bleachDelta.y)*correctionSat, 1.0), min(abs(bleachDelta.z)*2.0, 1.0));",
-				"	bleachLevel = b3.x*b3.y*b3.z;",
-				"	bleachLevel = max(bleachLevel, (abs(bleachDelta.y)*correctionSat+(1.0-correctionSat*2.0))*b3.z);",
+				"		vec3 b3 = vec3_111 - vec3(min(bleachLevel, 1.0), min(abs(bleachDelta.y)*correctionSat, 1.0), min(abs(bleachDelta.z)*2.0, 1.0));",
+				"		bleachLevel = b3.x*b3.y*b3.z;",
+				"		bleachLevel = max(bleachLevel, (abs(bleachDelta.y)*correctionSat+(1.0-correctionSat*2.0))*b3.z);",
 				"",
-				"	a = (1.0-correctionAlpha*bleachLevel) * a * color.a;",
+				"		a = (1.0-correctionAlpha*bleachLevel) * a * color.a;",
 			}));
 
 		if (maskOnly) {
 			list.addAll(Arrays.asList(new String[] {
-					"	gl_FragColor = vec4(a, a, a, 1.0);",
-					"}"
+				"		gl_FragColor = vec4(a, a, a, 1.0);",
+				"	} else {",
+				"		gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);",
+				"	}",
+				"}"
 			}));
 		} else {
 			list.addAll(Arrays.asList(new String[] {
-					"	unmult = clamp((unmult-bleachLevel*bleachRGB)/(1.0-bleachLevel), 0.0, 1.0);",
-					"	gl_FragColor = vec4(unmult, 1.0) * a;",
-					"}"
+				"		unmult = clamp((unmult-bleachLevel*bleachRGB)/(1.0-bleachLevel), 0.0, 1.0);",
+				"		gl_FragColor = vec4(unmult, 1.0) * a;",
+				"	} else {",
+				"		gl_FragColor = vec4(0.0);",
+				"	}",
+				"}"
 			}));
 		}
 
